#include "wseries.h"
#include "wdaubechies.h"

void decompose2d(double **a, int s1, int s2,int wo)
{
  int s12=s1/2;
  int s22=s2/2;
  WSeries wsr(s1),wsr1(s12),wsr2(s12);
  WDaubechies *waveL=new WDaubechies(wo);
  for(int i=0;i<s2;i++)
    {
      for(int j=0;j<s1;j++) wsr.data[j]=a[i][j];
      waveL->t2w(wsr,1);
      waveL->getLayer(wsr1,0);
      waveL->getLayer(wsr2,1);
      for(int j=0;j<s12;j++)
	{
	  a[i][j]=wsr1.data[j];
	  a[i][j+s12]=wsr2.data[j];
	}
    }
  wsr.Resize(s2);
  for(int j=0;j<s1;j++)
    {
      for(int i=0;i<s2;i++) wsr.data[i]=a[i][j];
      waveL->t2w(wsr,1);
      waveL->getLayer(wsr1,0);
      waveL->getLayer(wsr2,1);
      for(int i=0;i<s22;i++)
	{
	  a[i][j]=wsr1.data[i];
	  a[i+s22][j]=wsr2.data[i];
	}
    }
  delete waveL;
}

void i2w(double **a,int s1,int s2,int ldeep,int wo)
{
  int l1=s1;
  int l2=s2;
  for(int i=0;i<ldeep;i++)
    {
      decompose2d(a,l1,l2,wo);
      l1/=2;
      l2/=2;
    }
}

void reconstruct2d(double **a, int s1, int s2,int wo)
{
  int s12=s1/2;
  int s22=s2/2;
  WSeries wsr(s1),wsr1(s12),wsr2(s12);
  WDaubechies *waveL=new WDaubechies(s1,wo,0);
  for(int i=0;i<s2;i++)
    {
      for(int j=0;j<s12;j++)
	{
	  wsr1.data[j]=a[i][j];
	  wsr2.data[j]=a[i][j+s12];
	}
      waveL->Level=1;
      waveL->getMaxLevel();
      waveL->putLayer(wsr1,0);
      waveL->putLayer(wsr2,1);
      waveL->w2t(wsr,1);
      for(int j=0;j<s1;j++)
	{
	  a[i][j]=wsr.data[j];
	}
    }
  delete waveL;

  wsr.Resize(s2);
  wsr1.Resize(s22);
  wsr2.Resize(s22);
  waveL=new WDaubechies(s2,wo,0);
  for(int j=0;j<s1;j++)
    {
      for(int i=0;i<s22;i++)
	{
	  wsr1.data[i]=a[i][j];
	  wsr2.data[i]=a[i+s22][j];
	}
      waveL->Level=1;
      waveL->getMaxLevel();
      waveL->putLayer(wsr1,0);
      waveL->putLayer(wsr2,1);
      waveL->w2t(wsr,1);
      for(int i=0;i<s2;i++)
	{
	  a[i][j]=wsr.data[i];
	}
    }
  delete waveL;
}

void w2i(double **a,int s1,int s2,int ldeep,int wo)
{
  int l1=s1>>(ldeep-1);
  int l2=s2>>(ldeep-1);
  for(int i=0;i<ldeep;i++)
    {
      reconstruct2d(a,l1,l2,wo);
      l1*=2;
      l2*=2;
    }
}
