#include<stdio.h>
#include<stdlib.h>
#include<iostream.h>
#include <math.h>
#include <stream.h>
#include <fstream.h>
#include <complex.h>
//#include"ft.h"
//#include"ft.cc"

#include"Alt.h"
using namespace Alt;

union pix {
  unsigned char rgb[4];
  unsigned int val;
};
// FFT related functions 
double cdabs(complex<double> z)
{
double x,y;
x = real(z);
y = imag(z);
return sqrt(x*x+y*y);
}

/* All filters applied on Fourier transform imply that the Fourier 
   transform is symmetric with respect to the central point, i.e. 
   the field data[0..N-1]  corresponds to 
   [0, dk, 2dk, ... k_max,-k_max,-k_max+dk, ..., -dk] 
   and the same in Y-direction. 
   So, any filtering function is applied symmetricaly to 4 quandrants:
   Symbolically

   ++, -+
   +-, --

The scale parameter "a" is the dilatation factor in wavelet coefficients:
image[k] = filter_function[ak] * data[k]
*/ 
int Alt::filterMH(complex<double> *data, complex<double> *image,int xsize,int ysize, double a)
{
  // Mexican Hat filter 
double r2, v;
long dkx, dky, dk2, shift1, shift2;
int ys, xs;
ys = ysize/2;
xs = xsize/2;
// y=0
for(int k=1;k<xs;k++) 
  {
    r2 = a*a*k*k;
    v  = r2*exp(-0.5*r2);
    image[k] = v*data[k];
    image[xsize-k] = v*data[xsize-k];
  }// x=0
for(int k=1;k<ys;k++) 
  {
    int sh =         k*xsize;
    int sh1= (ysize-k)*xsize;
    r2 = a*a*k*k;
    v  = r2*exp(-0.5*r2);
    image[sh] = v*data[sh];
    image[sh1]= v*data[sh1];
  }
//y>0
for(int l=1;l<ys;l++)
  {
    shift1 = l*xsize;
    shift2 = (ysize-l)*xsize;
    dky = l;
    dk2 = dky*dky;
    for(int k=1;k<xs;k++)
      {
	int adr1 = k+shift1;           // (x,y)
	int adr2 = xsize - k + shift1; // (xsize-x,y)
	int adr3 = k+shift2;          // (x,ysize-y)
	int adr4 = xsize - k + shift2;// (xsize-x,ysize-y)
	dkx = k;
	r2  = dkx*dkx+dk2;
	r2  = a*a*r2;
	v   = r2*exp(-0.5*r2);
	image[adr1] = v*data[adr1];
	image[adr2] = v*data[adr2];
	image[adr3] = v*data[adr3];
	image[adr4] = v*data[adr4];
      }
  }
return 0;
}

int Alt::filterMorlet(complex<double> *data, complex<double> *image,int xsize,int ysize, double kx, double ky, double a)
{
  // complex Morlet wavelet k_0 = (kx,ky)
  // psi_M(k) = exp(-(k_0-k)^2/2)  
double r2, v, rx, ry ,dk2;
long shift1, shift2;
int ys, xs;
ys = ysize/2;
xs = xsize/2;
// y=0
image[0] = exp(-0.5*(kx*kx+ky*ky))*data[0];
for(int k=1;k<xs;k++) 
  {
    rx = a*k-kx;
    r2 = rx*rx+ky*ky;
    v  = exp(-0.5*r2);
    image[k] = v*data[k];
    image[xsize-k] = v*data[xsize-k];
  }// x=0
for(int k=1;k<ys;k++) 
  {
    int sh =         k*xsize;
    int sh1= (ysize-k)*xsize;
    ry = a*k-ky;
    r2 = ry*ry+kx*kx;
    v  = exp(-0.5*r2);
    image[sh] = v*data[sh];
    image[sh1]= v*data[sh1];
  }
//y>0
for(int l=1;l<ys;l++)
  {
    shift1 = l*xsize;
    shift2 = (ysize-l)*xsize;
    ry  = a*l-ky;
    
    dk2 = ry*ry;
    for(int k=1;k<xs;k++)
      {
	int adr1 = k+shift1;           // (x,y)
	int adr2 = xsize - k + shift1; // (xsize-x,y)
	int adr3 = k+shift2;          // (x,ysize-y)
	int adr4 = xsize - k + shift2;// (xsize-x,ysize-y)
	rx = a*k-kx;
	r2  = rx*rx+dk2;
	v   = exp(-0.5*r2);
	image[adr1] = v*data[adr1];
	image[adr2] = v*data[adr2];
	image[adr3] = v*data[adr3];
	image[adr4] = v*data[adr4];
      }
  }
return 0;
}

/* n-dimensional FFT algorithm from Numerical Recipies. 
   Because the code was modified from FORTRAN source, the 
   first element data[0] is not used. The further arrangement 
   of the data set are 
   Re1,Im1, Re2,Im2, Re3,Im3, and so on 
   The size of each dimension is describe in nn[] 
   nn[0] - nothing
   nn[1] - Xsize
   nn[2] - Ysize 
   ... 
   ndim - is the number of dimensions , ndim=2 for 2D FFT
   isign = 1, direct FFT , =-1 Inverse FFT 
   The result of transform substitute the original data on exit
*/

#define SWAP(a,b) tempr=(a);(a)=(b);(b)=tempr

void Alt::fourn(double data[], unsigned long nn[], int ndim, int isign)
{
	int idim;
	unsigned long i1,i2,i3,i2rev,i3rev,ip1,ip2,ip3,ifp1,ifp2;
	unsigned long ibit,k1,k2,n,nprev,nrem,ntot;
	double tempi,tempr;
	double theta,wi,wpi,wpr,wr,wtemp;

	for (ntot=1,idim=1;idim<=ndim;idim++)
		ntot *= nn[idim];
	nprev=1;
	for (idim=ndim;idim>=1;idim--) {
		n=nn[idim];
		nrem=ntot/(n*nprev);
		ip1=nprev << 1;
		ip2=ip1*n;
		ip3=ip2*nrem;
		i2rev=1;
		for (i2=1;i2<=ip2;i2+=ip1) {
			if (i2 < i2rev) {
				for (i1=i2;i1<=i2+ip1-2;i1+=2) {
					for (i3=i1;i3<=ip3;i3+=ip2) {
						i3rev=i2rev+i3-i2;
						SWAP(data[i3],data[i3rev]);
						SWAP(data[i3+1],data[i3rev+1]);
					}
				}
			}
			ibit=ip2 >> 1;
			while (ibit >= ip1 && i2rev > ibit) {
				i2rev -= ibit;
				ibit >>= 1;
			}
			i2rev += ibit;
		}
		ifp1=ip1;
		while (ifp1 < ip2) {
			ifp2=ifp1 << 1;
			theta=isign*6.28318530717959/(ifp2/ip1);
			wtemp=sin(0.5*theta);
			wpr = -2.0*wtemp*wtemp;
			wpi=sin(theta);
			wr=1.0;
			wi=0.0;
			for (i3=1;i3<=ifp1;i3+=ip1) {
				for (i1=i3;i1<=i3+ip1-2;i1+=2) {
					for (i2=i1;i2<=ip3;i2+=ifp2) {
						k1=i2;
						k2=k1+ifp1;
						tempr=(double)wr*data[k2]-(double)wi*data[k2+1];
						tempi=(double)wr*data[k2+1]+(double)wi*data[k2];
						data[k2]=data[k1]-tempr;
						data[k2+1]=data[k1+1]-tempi;
						data[k1] += tempr;
						data[k1+1] += tempi;
					}
				}
				wr=(wtemp=wr)*wpr-wi*wpi+wr;
				wi=wi*wpr+wtemp*wpi+wi;
			}
			ifp1=ifp2;
		}
		nprev *= n;
	}
}
#undef SWAP
