/***************************************************************************
    begin                : Wed Apr 10 2002
    copyright            : (C) 2002 by Alexei Soloviev
    email                : solovjev@lynx.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include<algorithm>
#include<cmath>

#include"DM.h"
#include"ROD.h"
#include"Signal.h"


DM::DM() : 
  event(0), curr_state(1), 
  name(), x_name(), y_name(), 
  x_min(0), x_max(0), y_min(0), y_max(0), 
  fixed(false) 
{}

DM &DM::operator<<(const Event &e)
{
  event=&e;
  return *this;
}

DM &DM::operator<<(const int &s)
{
  if( s>0 && s<7 ) curr_state=s;
  return *this;
}

Sample DM::sample() const
{
  Sample s;

  for(Event::const_iterator p=event->begin(); p!=event->end(); p++) {
    switch(curr_state) {
    case 1: s.push_back(p->eta()); break;
    case 2: s.push_back(p->theta()); break;
    case 3: s.push_back(p->phi()); break;
    default: break;
    }
  }

  return s;
}

H1 DM::hist_1(int resolution=32) const
{
  if(!event)throw No_Event();

  int xbins=resolution;
  std::string title, xtitle, ytitle;
  double xmin, xmax;

  switch(curr_state) {
  case 1:
    title = fixed ? name : "#eta distribution";
    xmin = fixed ? x_min : event->min_eta();
    xmax = fixed ? x_max : event->max_eta();
    break;
  case 2:
    title = fixed ? name : "#theta distribution";
    xmin = fixed ? x_min : event->min_theta();
    xmax = fixed ? x_max : event->max_theta();
    break;
  case 3:
    title = fixed ? name : "#phi distribution";
    xmin = fixed ? x_min : event->min_phi();
    xmax = fixed ? x_max : event->max_phi();
    break;
  default: 
    break;
  }

  xtitle = fixed ? x_name : "";
  ytitle = fixed ? y_name : "";

  H1 h(xbins,xmin,xmax,title.c_str(),xtitle.c_str(),ytitle.c_str());

  double w=1/h.xstep()/event->size();

  for(Event::const_iterator p=event->begin(); p!=event->end(); p++) {
    switch(curr_state) {
    case 1: h.fill(p->eta(),w); break;
    case 2: h.fill(p->theta(),w); break;
    case 3: h.fill(p->phi(),w); break;
    default: break;
    }
  }

  return h;
}

H2 DM::hist_2(int resolution=32) const
{
  if(!event)throw No_Event();

  int xbins=resolution, ybins=resolution;
  std::string title, xtitle, ytitle;
  double xmin, xmax, ymin, ymax;

  switch(curr_state) {
  case 4:
    title = fixed ? name : "Target diagram, sin( #theta )";
    xmin = fixed ? x_min : -1;
    xmax = fixed ? x_max : 1;
    ymin = fixed ? y_min : -1;
    ymax = fixed ? y_max : 1;
    break;
  case 5:
    title = fixed ? name : "Target diagram, tg( #theta )";
    xmin = fixed ? x_min : -1.7320508076;  // -tg(pi/3)
    xmax = fixed ? x_max : 1.7320508076;  // tg(pi/3)
    ymin = fixed ? y_min : -1.7320508076;  // -tg(pi/3)
    ymax = fixed ? y_max : 1.7320508076;  // tg(pi/3)
    break;
  case 6:
    title = fixed ? name : "#phi - #eta plot";
    xmin = fixed ? x_min : event->min_eta();
    xmax = fixed ? x_max : event->max_eta();
    ymin = fixed ? y_min : event->min_phi();
    ymax = fixed ? y_max : event->max_phi();
    break;
  default: 
    break;
  }

  xtitle = fixed ? x_name : "";
  ytitle = fixed ? y_name : "";

  H2 h(xbins,xmin,xmax,ybins,ymin,ymax,title.c_str(),xtitle.c_str(),ytitle.c_str());

  for(Event::const_iterator p=event->begin(); p!=event->end(); p++) {
    switch(curr_state) {
    case 4: 
      h.fill(sin(p->theta()) * cos(p->phi()), sin(p->theta()) * sin(p->phi())); 
      break;
    case 5: 
      h.fill(tan(p->theta()) * cos(p->phi()), tan(p->theta()) * sin(p->phi())); 
      break;
    case 6: h.fill(p->eta(), p->phi()); break;
    default: break;
    }
  }

  return h;
}

void DM::use_parm()
{
  fixed=true;
}

void DM::clear_parm()
{
  fixed=false;
  set_title(); set_xtitle(); set_ytitle();
  set_xmin(); set_xmax(); set_ymin(); set_ymax();
}
