/***************************************************************************
    begin                : Wed Apr 10 2002
    copyright            : (C) 2002 by Alexei Soloviev
    email                : solovjev@lynx.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LRCD_DM_H
#define LRCD_DM_H

#include<string>
#include<list>

typedef std::list<double> Sample;

class H1;
class H2;
class Event;


class DM  // Distributions Maker
{
  const Event *event;

  int curr_state;

  // fixed distribution parameters:
  std::string name, x_name, y_name;
  double x_min, x_max, y_min, y_max;
  bool fixed;

 public:
  DM();
  ~DM(){}

  DM &operator<<(const Event &e);
  DM &operator<<(const int &s);

  inline int state() const { return curr_state; }

  Sample sample() const;

  H1 hist_1(int resolution=32) const;
  H2 hist_2(int resolution=32) const;

  // fixed distribution parameters:
  inline const bool is_fixed() const { return fixed; }

  inline const char *title() const { return name.c_str(); }
  inline const char *xtitle() const { return x_name.c_str(); }
  inline const char *ytitle() const { return y_name.c_str(); }

  inline double xmin() const { return x_min; }
  inline double xmax() const { return x_max; }
  inline double ymin() const { return y_min; }
  inline double ymax() const { return y_max; }

  void use_parm();
  void clear_parm();

  inline void set_title(const char *t="") { name=t; }
  inline void set_xtitle(const char *xt="") { x_name=xt; }
  inline void set_ytitle(const char *yt="") { y_name=yt; }

  inline void set_xmin(double x=0) { x_min=x; }
  inline void set_xmax(double x=0) { x_max=x; }
  inline void set_ymin(double y=0) { y_min=y; }
  inline void set_ymax(double y=0) { y_max=y; }
};

#endif  // LRCD_DM_H
