/***************************************************************************
    begin                : Wed Apr 10 2002
    copyright            : (C) 2002 by Alexei Soloviev
    email                : solovjev@lynx.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include<algorithm>
#include<cmath>

#include"DV.h"
#include"Signal.h"


DV::DV(const char *name1="h1", const char *name2="h2") : 
  pad(0), option(), grays() 
{
  hist1=new TH1D();
  hist2=new TH2D();

  hist1->SetName(name1);
  hist2->SetName(name2);

  hist1->SetStats(false);  // don't draw statistics box
  hist2->SetStats(false);  // don't draw statistics box

  hist1->SetMinimum();
  hist1->SetMaximum();

  hist2->SetMarkerStyle(4);  // open circle marker (used when option="")
  hist2->SetMarkerSize(0.2);  // small marker (used when option="")
}


DV::~DV()
{
  delete hist1;
  delete hist2;
}

void DV::associate(TPad *p)
{
  if((pad=p)) {
    pad->cd();
    hist1->SetFillColor(pad->GetFillColor());
  }
}

DV &DV::operator<<(const H1 &h)
{
  if(!pad) return *this;
  pad->Clear();

  if(h.xmin()>=h.xmax()||!h.size()) return *this;

  hist1->SetBins(h.size(), h.xmin(), h.xmax());

  for(unsigned int i=0; i<h.size(); i++) hist1->SetBinContent(i+1, h[i]);

  hist1->Draw(option.c_str());

  hist1->SetTitle(h.title());
  hist1->GetXaxis()->SetTitle(h.xtitle());
  hist1->GetYaxis()->SetTitle(h.ytitle());

  pad->Update();

  return *this;
}

DV &DV::operator<<(const H2 &h)
{
  if(!pad) return *this;
  pad->Clear();

  if(h.xmin()>=h.xmax()||h.ymin()>=h.ymax()||!h.size())return *this;

  hist2->SetBins(h.xbins(), h.xmin(), h.xmax(), 
		 h.ybins(), h.ymin(), h.ymax());

  double MIN=0, MAX=0;

  for(int i=0; i<h.ybins(); i++) {
    for(int j=0; j<h.xbins(); j++) {
      hist2->SetCellContent(j+1,i+1, h[i][j]);
      MIN=min(MIN, h[i][j]);
      MAX=max(MAX, h[i][j]);
    }
  }

  MIN-=(MAX-MIN)*1e-10;  // bacause (MIN,MAX) is taking by TH2D
  MAX+=(MAX-MIN)*1e-10;  // bacause (MIN,MAX) is taking by TH2D

  hist2->SetMinimum(MIN);
  hist2->SetMaximum(MAX);

  hist2->Draw(option.c_str());

  hist2->SetTitle(h.title());
  hist2->GetXaxis()->SetTitle(h.xtitle());
  hist2->GetYaxis()->SetTitle(h.ytitle());

  pad->Update();

  return *this;
}

DV &DV::operator<<(const std::string &opt)
{
  option=opt;
  return *this;
}

DV &DV::operator<<(const char *opt)
{
  option=opt;
  return *this;
}

