/***************************************************************************
    begin                : Wed Apr 10 2002
    copyright            : (C) 2002 by Alexei Soloviev
    email                : solovjev@lynx.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include<algorithm>

#include"Event.h"


Entry::Entry(double THETA=0, double PHI=0) : 
  fTheta(THETA), fPhi(PHI) 
{
  while(fPhi < 0)    fPhi += pi*2;
  while(fPhi > pi*2) fPhi -= pi*2;
}

std::istream &operator>>(std::istream &i, Entry &e) 
{
  (i>>e.fTheta>>e.fPhi).ignore(1024,'\n');

  while(e.fPhi < 0)      e.fPhi += e.pi*2;
  while(e.fPhi > e.pi*2) e.fPhi -= e.pi*2;

  return i;
}

Event::Event() : 
  std::list<Entry>(), min_entry(), max_entry() 
{}

Event &Event::operator+=(const Entry &e)
{
  if(empty()) min_entry=max_entry=e;

  push_back(e);

  min_entry=Entry(min(min_theta(),e.theta()), min(min_phi(),e.phi()));
  max_entry=Entry(max(max_theta(),e.theta()), max(max_phi(),e.phi()));

  return *this;
}

Event &Event::operator+=(const Event &e)
{
  if(empty()) return *this=e;

  for(const_iterator p=e.begin(); p!=e.end(); p++)
    push_back(*p);

  min_entry=Entry(min(min_theta(),e.min_theta()), min(min_phi(),e.min_phi()));
  max_entry=Entry(max(max_theta(),e.max_theta()), max(max_phi(),e.max_phi()));

  return *this;
}

std::istream &operator>>(std::istream &i, Event &e)
{
  e.clear();

  Entry s;
//    if(!(i>>s)) {  // skip one lines without entries
  while(!(i>>s)) {  // skip all lines without entries
    i.clear();
    i.ignore(1024,'\n');
    if(i.eof()) return i;
  }
  do{ e+=s; } while(i>>s);

  if(!i.eof()) i.clear();

  return i;
}

