/***************************************************************************
    begin                : Wed Apr 10 2002
    copyright            : (C) 2002 by Alexei Soloviev
    email                : solovjev@lynx.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LRCD_Event_H
#define LRCD_Event_H

#include<iostream>
#include<list>
#include<cmath>


class Entry
{
 public:
  static const double pi=3.1415926536;

 private:
  double fTheta;  // polar angle, range is [0, pi]
  double fPhi;  // azimutal angle, range is [0, 2*pi]

 public:
  Entry(double THETA=0, double PHI=0);
  ~Entry() {}

  inline double theta() const { return fTheta; }
  inline double phi() const { return fPhi; }
  inline double eta() const {   // pseudorapidity = -log(tan(theta/2))
    return (fTheta >0 && fTheta < pi) ? -log(tan(fTheta/2)) : 0; 
  }

  friend std::istream &operator>>(std::istream &i, Entry &e);
};

class Event : public std::list<Entry>
{
  Entry min_entry;
  Entry max_entry;

 public:
  Event();
  ~Event() {}

  Event &operator+=(const Entry &e);
  Event &operator+=(const Event &e);

  inline double min_theta() const { return min_entry.theta(); }
  inline double min_phi() const { return min_entry.phi(); }
  inline double min_eta() const { return max_entry.eta(); }

  inline double max_theta() const { return max_entry.theta(); }
  inline double max_phi() const { return max_entry.phi(); }
  inline double max_eta() const { return min_entry.eta(); }

  friend std::istream &operator>>(std::istream &i, Event &e);
};

#endif  // LRCD_Event_H
