/***************************************************************************
    begin                : Wed Apr 10 2002
    copyright            : (C) 2002 by Alexei Soloviev
    email                : solovjev@lynx.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LRCD_Filter_H
#define LRCD_Filter_H

#include<algorithm>

#include"Event.h"


class Filter
{
 public:
  static const int n_min_default=0;
  static const int n_max_default=100000;
  static const Entry e_min_default;
  static const Entry e_max_default;

 private:
  unsigned int n_min, n_max;
  Entry e_min, e_max;

  bool on;

 public:
  Filter();
  ~Filter(){}

  inline bool test_event(const Event &e)
    {
      if(!on) return true;

      if(e.size() < n_min || e.size() > n_max) return false;
      if(e.min_theta() < e_min.theta() || e.max_theta() > e_max.theta()) 
	return false;
      if(e.min_phi() < e_min.phi() || e.max_phi() > e_max.phi()) 
	return false;

      return true;
    }

  inline bool is_on() const { return on; }

  inline int get_n_min() const { return n_min; }
  inline int get_n_max() const { return n_max; }
  inline Entry get_e_min() const { return e_min; }
  inline Entry get_e_max() const { return e_max; }

  void switch_on() { on=true; }
  void switch_off() { on=false; }

  void set_n_min(int n=n_min_default) { n_min=n; }
  void set_n_max(int n=n_max_default) { n_max=n; }
  void set_e_min(const Entry &e=e_min_default) { e_min=e; }
  void set_e_max(const Entry &e=e_max_default) { e_max=e; }

  void reset();
};

#endif  // LRCD_Filter_H
