/***************************************************************************
    begin                : Wed Apr 10 2002
    copyright            : (C) 2002 by Alexei Soloviev
    email                : solovjev@lynx.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include<cstdlib>

#include <TApplication.h>
#include <TGClient.h>
#include <TGString.h>

#include"FilterFrame.h"


FilterFrame::FilterFrame(Filter *f, const TGWindow *p, const TGWindow *main, 
			 UInt_t w, UInt_t h)
  : TGTransientFrame(p, main, w, h), filter(f) 
{
  // Dialog used to set the filter.

  char buff[10];

  fFrame[0]=new TGGroupFrame(this, "Multiplicity", kHorizontalFrame);
  fFrame[1]=new TGGroupFrame(this, "Polar angle", kHorizontalFrame);
  fFrame[2]=new TGGroupFrame(this, "Azimutal angle", kHorizontalFrame);

  fFL=new TGLayoutHints(kLHintsTop | kLHintsRight, 4, 4, 0, 4);

  for(int i=0; i<3; i++) {
    fLf1[i]=new TGCompositeFrame(fFrame[i], 60, 20, kVerticalFrame);
    fLf2[i]=new TGCompositeFrame(fFrame[i], 60, 20, kVerticalFrame);

    fLabel1[i]=new TGLabel(fLf1[i], "min");
    fLabel2[i]=new TGLabel(fLf2[i], "max");

    fTe1[i]=new TGTextEntry(fLf1[i], fTb1[i]=new TGTextBuffer(10), i+10);
    fTe2[i]=new TGTextEntry(fLf2[i], fTb2[i]=new TGTextBuffer(10), i+10);
  }

  sprintf(buff, "%d", filter->get_n_min());
  fTb1[0]->AddText(0, buff);  // min
  sprintf(buff, "%d", filter->get_n_max());
  fTb2[0]->AddText(0, buff);  // max
  sprintf(buff, "%f", filter->get_e_min().theta());
  fTb1[1]->AddText(0, buff);  // min
  sprintf(buff, "%f", filter->get_e_max().theta());
  fTb2[1]->AddText(0, buff);  // max
  sprintf(buff, "%f", filter->get_e_min().phi());
  fTb1[2]->AddText(0, buff);  // min
  sprintf(buff, "%f", filter->get_e_max().phi());
  fTb2[2]->AddText(0, buff);  // max

  for(int i=0; i<3; i++) {
    fTe1[i]->Associate(this);
    fTe2[i]->Associate(this);
  }

  fLL=new TGLayoutHints(kLHintsTop | kLHintsCenterX | kLHintsExpandX, 2, 2, 0, 0);

  for(int i=0; i<3; i++) {
    fLf1[i]->AddFrame(fLabel1[i], fLL);
    fLf1[i]->AddFrame(fTe1[i], fLL);
    fLf2[i]->AddFrame(fLabel2[i], fLL);
    fLf2[i]->AddFrame(fTe2[i], fLL);

    fLf1[i]->Resize(100,100);
    fLf2[i]->Resize(100,100);
  }

  for(int i=0; i<3; i++) {
    fSlider[i]=new TGDoubleHSlider(fFrame[i], 200, kDoubleScaleBoth, i+100);
    fSlider[i]->Associate(this);
  }

  fSlider[0]->SetRange(filter->n_min_default,filter->n_max_default);
  fSlider[0]->SetPosition(filter->get_n_min(),filter->get_n_max());

  fSlider[1]->SetRange(filter->e_min_default.theta(),filter->e_max_default.theta());
  fSlider[1]->SetPosition(filter->get_e_min().theta(),filter->get_e_max().theta());

  fSlider[2]->SetRange(filter->e_min_default.phi(),filter->e_max_default.phi());
  fSlider[2]->SetPosition(filter->get_e_min().phi(),filter->get_e_max().phi());

  //--- layout for buttons:
  fL = new TGLayoutHints(kLHintsBottom | kLHintsLeft, 2, 2, 2, 0);

  for(int i=0; i<3; i++) {
    fFrame[i]->AddFrame(fLf1[i], fL);
    fFrame[i]->AddFrame(fSlider[i], fL);
    fFrame[i]->AddFrame(fLf2[i], fL);

    AddFrame(fFrame[i], fFL);
  }

  // Create status frame containing buttons and text entriy widgets
  fStatusFrame = new TGCompositeFrame(this, 60, 20, kHorizontalFrame |
				      kSunkenFrame);

  fOnButton = new TGTextButton(fStatusFrame, " On ", 151);
  fOnButton->Associate(this);
  fOnButton->SetToolTipText("Switch filter on");
  fStatusFrame->AddFrame(fOnButton, new TGLayoutHints(kLHintsTop |
						      kLHintsLeft, 2, 2, 2, 2));
  fOffButton = new TGTextButton(fStatusFrame, " Off ", 152);
  fOffButton->Associate(this);
  fOffButton->SetToolTipText("Switch filter off");
  fStatusFrame->AddFrame(fOffButton, new TGLayoutHints(kLHintsTop |
						      kLHintsLeft, 2, 2, 2, 2));
  fResetButton = new TGTextButton(fStatusFrame, " Reset ", 153);
  fResetButton->Associate(this);
  fResetButton->SetToolTipText("Reset filter");
  fStatusFrame->AddFrame(fResetButton, new TGLayoutHints(kLHintsTop |
							 kLHintsLeft, 2, 2, 2, 2));

  fStatusLabel=new TGLabel(fStatusFrame, 
			   (filter->is_on()?"Filter is on":"Filter is off"));
  fStatusFrame->AddFrame(fStatusLabel, new TGLayoutHints(kLHintsBottom |
							 kLHintsCenterX, 2, 2, 2, 2));


  fOkButton = new TGTextButton(fStatusFrame, "   Ok   ", 155);
  fOkButton->Associate(this);
  fOkButton->SetToolTipText("Accept these settings");
  fStatusFrame->AddFrame(fOkButton, new TGLayoutHints(kLHintsTop |
						      kLHintsRight, 2, 2, 2, 2));
  fCancelButton = new TGTextButton(fStatusFrame, " Cancel ", 154);
  fCancelButton->Associate(this);
  fCancelButton->SetToolTipText("Close filter window");
  fStatusFrame->AddFrame(fCancelButton, new TGLayoutHints(kLHintsTop |
							  kLHintsRight, 2, 2, 2, 2));
  AddFrame(fStatusFrame, new TGLayoutHints(kLHintsBottom | kLHintsExpandX,
					   0, 0, 1, 0));


  SetWindowName("Filter settings");
  TGDimension size = GetDefaultSize();
  Resize(size);

  SetWMSize(size.fWidth, size.fHeight);
  SetWMSizeHints(size.fWidth, size.fHeight, size.fWidth, size.fHeight, 0, 0);
  SetMWMHints(kMWMDecorAll | kMWMDecorResizeH  | kMWMDecorMaximize |
	      kMWMDecorMinimize | kMWMDecorMenu,
	      kMWMFuncAll |  kMWMFuncResize    | kMWMFuncMaximize |
	      kMWMFuncMinimize,
	      kMWMInputModeless);

  // position relative to the parent's window
  Window_t wdummy;
  int ax, ay;
  gVirtualX->TranslateCoordinates(main->GetId(), GetParent()->GetId(),
				  (((TGFrame *) main)->GetWidth() - fWidth) >> 1,
				  (((TGFrame *) main)->GetHeight() - fHeight) >> 1,
				  ax, ay, wdummy);
  Move(ax, ay);

  MapSubwindows();
  MapWindow();

  fClient->WaitFor(this);
}

FilterFrame::~FilterFrame()
{
  delete fFrame[0]; delete fFrame[1]; delete fFrame[2];
  delete fSlider[0]; delete fSlider[1]; delete fSlider[2];

  delete fLabel1[0]; delete fLabel2[0];
  delete fLabel1[1]; delete fLabel2[1];
  delete fLabel1[2]; delete fLabel2[2];

  delete fLf1[0]; delete fLf2[0];
  delete fTe1[0]; delete fTe2[0];
  delete fLf1[1]; delete fLf2[1];
  delete fTe1[1]; delete fTe2[1];
  delete fLf1[2]; delete fLf2[2];
  delete fTe1[2]; delete fTe2[2];

  delete fFL; delete fL; delete fLL;

  delete fStatusFrame;
  delete fOnButton; delete fOffButton; delete fResetButton;
  delete fCancelButton; delete fOkButton;
  delete fStatusLabel;
}

void FilterFrame::CloseWindow()
{
  // Called when window is closed via the window manager.

  delete this;
}

Bool_t FilterFrame::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2)
{
  // Process slider messages.

  char buff[10];

  switch (GET_MSG(msg)) {

    // button was pressed:
  case kC_COMMAND:
    switch (GET_SUBMSG(msg)) {
    case kCM_BUTTON:
      switch(parm1){
      case 151: // On
	filter->switch_on();
	fStatusLabel->SetText(new TGString("Filter is on"));
	fClient->NeedRedraw(fStatusLabel);
	break;
      case 152: // Off
	filter->switch_off();
	fStatusLabel->SetText(new TGString("Filter is off"));
	fClient->NeedRedraw(fStatusLabel);
	break;
      case 153: // Reset
	filter->reset();
	fSlider[0]->SetPosition(filter->get_n_min(), filter->get_n_max());
	fSlider[1]->SetPosition(filter->get_e_min().theta(), 
				filter->get_e_max().theta());
	fSlider[2]->SetPosition(filter->get_e_min().phi(), 
				filter->get_e_max().phi());

	for(int i=0; i<3; i++) {
	  fTb1[i]->Clear();
	  fTb2[i]->Clear();
	}

	sprintf(buff, "%d", filter->get_n_min());
	fTb1[0]->AddText(0, buff); // min
	sprintf(buff, "%d", filter->get_n_max());
	fTb2[0]->AddText(0, buff); // max

	sprintf(buff, "%f", filter->get_e_min().theta());
	fTb1[1]->AddText(0, buff); // min
	sprintf(buff, "%f", filter->get_e_max().theta());
	fTb2[1]->AddText(0, buff); // max

	sprintf(buff, "%f", filter->get_e_min().phi());
	fTb1[2]->AddText(0, buff); // min
	sprintf(buff, "%f", filter->get_e_max().phi());
	fTb2[2]->AddText(0, buff); // max

	for(int i=0; i<3; i++) {
	  fClient->NeedRedraw(fTe1[i]);
	  fClient->NeedRedraw(fTe2[i]);
	}

	fStatusLabel->SetText(new TGString("Filter is off"));
	fClient->NeedRedraw(fStatusLabel);
	break;
      case 154: // Cancel
	CloseWindow();
	break;
      case 155: // Ok
	filter->set_n_min(atoi(fTb1[0]->GetString()));
	filter->set_n_max(atoi(fTb2[0]->GetString()));
	filter->set_e_min(Entry(atof(fTb1[1]->GetString()),
				atof(fTb1[2]->GetString())));
	filter->set_e_max(Entry(atof(fTb2[1]->GetString()),
				atof(fTb2[2]->GetString())));
	CloseWindow();
	break;
      }
      break;
    }
    break;

  case kC_TEXTENTRY:
    switch (GET_SUBMSG(msg)) {
    case kTE_TEXTCHANGED:
      switch (parm1) {
      case 10:
	fSlider[0]->SetPosition(atoi(fTb1[0]->GetString()),
				atoi(fTb2[0]->GetString()));
	break;
      case 11:
	fSlider[1]->SetPosition(atof(fTb1[1]->GetString()),
				atof(fTb2[1]->GetString()));
	break;
      case 12:
	fSlider[2]->SetPosition(atof(fTb1[2]->GetString()),
				atof(fTb2[2]->GetString()));
	break;
      }
      break;
    }
    break;
  case kC_HSLIDER:
    switch (GET_SUBMSG(msg)) {
    case kSL_POS:
      sprintf(buff, "%ld", parm2);
      switch (parm1) {
      case 100:
	sprintf(buff, "%d", (int)fSlider[0]->GetMinPosition());
	fTb1[0]->Clear();
	fTb1[0]->AddText(0, buff); // min
	fClient->NeedRedraw(fTe1[0]);
	sprintf(buff, "%d", (int)fSlider[0]->GetMaxPosition());
	fTb2[0]->Clear();
	fTb2[0]->AddText(0, buff); // max
	fClient->NeedRedraw(fTe2[0]);
	break;
      case 101:
	sprintf(buff, "%f", fSlider[1]->GetMinPosition());
	fTb1[1]->Clear();
	fTb1[1]->AddText(0, buff); // min
	fClient->NeedRedraw(fTe1[1]);
	sprintf(buff, "%f", fSlider[1]->GetMaxPosition());
	fTb2[1]->Clear();
	fTb2[1]->AddText(0, buff); // max
	fClient->NeedRedraw(fTe2[1]);
	break;
      case 102:
	sprintf(buff, "%f", fSlider[2]->GetMinPosition());
	fTb1[2]->Clear();
	fTb1[2]->AddText(0, buff); // min
	fClient->NeedRedraw(fTe1[2]);
	sprintf(buff, "%f", fSlider[2]->GetMaxPosition());
	fTb2[2]->Clear();
	fTb2[2]->AddText(0, buff); // max
	fClient->NeedRedraw(fTe2[2]);
	break;
      }
      break;
    }
    break;
  default:
    break;
  }
  return kTRUE;
}

