/***************************************************************************
    begin                : Wed Apr 10 2002
    copyright            : (C) 2002 by Alexei Soloviev
    email                : solovjev@lynx.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LRCD_GWT_H
#define LRCD_GWT_H

#include<cmath>
#include<list>
#include<vector>
#include<string>


typedef std::list<double> Sample;

class H1;
class H2;


class GWT  // Gaussian Wavelet Transformer
{
  inline static double gauss(double x){ return exp(-x*x/2); }

  // Wavalets:
  inline static double g1wavelet(double x)
    { return -x*gauss(x); }
  inline static double g2wavelet(double x)
    { return (1-x*x)*gauss(x); }
  inline static double g3wavelet(double x)
    { return (3-x*x)*x*gauss(x); }
  inline static double g4wavelet(double x)
    { return ((6-x*x)*x*x-3)*gauss(x); }
  inline static double g5wavelet(double x)
    { return ((10-x*x)*x*x-15)*x*gauss(x); }
  inline static double g6wavelet(double x)
    { return (((15-x*x)*x*x-45)*x*x+15)*gauss(x); }

  static const double g1coefsq=2.5066282746310005;
  static const double g2coefsq=2.5066282746310005;
  static const double g3coefsq=3.544907701811032;
  static const double g4coefsq=6.1399602476789308;
  static const double g5coefsq=12.279920495357862;
  static const double g6coefsq=19.416259125556991;

  int used_wavelet;
  bool even;

  // fixed wavelet spectrum parameters:
  std::string name, s_name;
  double s_min, s_max;
  bool fixed;

  // Wavelet values table:
  std::vector<double> wavelet_table(int size, double normcoef, const double &scalecoef);

 public:
  GWT();
  ~GWT(){}

  GWT &operator<<(const int &w);

  inline int wavelet() const { return used_wavelet; }

  // Histogram transformations:
  H1 transform(const H1 &h, const double &scale);
  H2 transform(const H1 &h, int s_bins, const double &s_min, const double &s_max);
  H2 transform(const H1 &h);  // default parameters

  H2 transform(const H2 &h, double scale);
  H2 transform(const H2 &h, int level, int throw_off=0);

  // Sample transformations:
  double transform(const Sample &s, const double &scale, const double &x);

  H1 transform(const Sample &s, const double &scale, const H1 *pattern=0);
  H2 transform(const Sample &s, 
	       int s_bins, const double &s_min, const double &s_max, 
	       const H1 *pattern=0);
  H2 transform(const Sample &s, const H1 *pattern=0);  // default parameters

  // fixed distribution parameters:
  inline const bool is_fixed() const { return fixed; }

  inline const char *title() const { return name.c_str(); }
  inline const char *ytitle() const { return s_name.c_str(); }

  inline double ymin() const { return s_min; }
  inline double ymax() const { return s_max; }

  void use_parm();
  void clear_parm();

  inline void set_title(const char *t="") { name=t; }
  inline void set_ytitle(const char *yt="") { s_name=yt; }

  inline void set_ymin(double y=0) { s_min=y; }
  inline void set_ymax(double y=0) { s_max=y; }
};

#endif   // LRCD_GWT_H

