/***************************************************************************
    begin                : Wed Apr 10 2002
    copyright            : (C) 2002 by Alexei Soloviev
    email                : solovjev@lynx.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <TApplication.h>
#include <TVirtualX.h>
#include <TGClient.h>

#include <TGMsgBox.h>

#include"GWTFrame.h"
#include"GWT.h"


GWTFrame::GWTFrame(GWT *t, const TGWindow *p, const TGWindow *main,
		   UInt_t w, UInt_t h, UInt_t options) :
  TGTransientFrame(p, main, w, h, options), wt(t)
{
  // Create message box dialog. 

  //------------------------------
  // Create new graphics context for drawing of TGlabel and TGButtons
  // with text in red.

  // Use default font as specified in .rootrc
  FontStruct_t labelfont;
  labelfont = fClient->GetFontByName(gEnv->GetValue("Gui.NormalFont",
               "-adobe-helvetica-medium-r-*-*-12-*-*-*-*-*-iso8859-1"));

  // Define new graphics context. Only the fields specified in
  // fMask will be used (for default TGLabel context see
  // http://root.cern.ch/root/html/src/TGClient.cxx.html).
  GCValues_t   gval;
  gval.fMask = kGCForeground | kGCFont;
  gval.fFont = gVirtualX->GetFontHandle(labelfont);
  fClient->GetColorByName("red", gval.fForeground);

  fRedTextGC = gVirtualX->CreateGC(fClient->GetRoot()->GetId(), &gval);
  //---------------------------------

   int i, ax, ay;

   // Layouts
   fL1 = new TGLayoutHints(kLHintsTop | kLHintsExpandX, 4, 4, 2, 2);

   fL2 = new TGLayoutHints(kLHintsBottom | kLHintsRight, 2, 2, 2, 2);

   fL3 = new TGLayoutHints(kLHintsTop | kLHintsLeft |
                           kLHintsExpandX | kLHintsExpandY, 4, 4, 2, 2);

   fL4 = new TGLayoutHints(kLHintsTop | kLHintsLeft, 0, 0, 2, 0);

   fL5 = new TGLayoutHints(kLHintsLeft | kLHintsCenterY, 4, 4, 0, 0);
   fL6 = new TGLayoutHints(kLHintsRight | kLHintsCenterY, 4, 4, 0, 0);

   // Info frame
   fInfoFrame = new TGCompositeFrame(this, 60, 20, kHorizontalFrame);

   fLtitle = new TGLabel(fInfoFrame, new TGString("Current wavelet:"), 
			 fRedTextGC);
   fTitle = new TGTextEntry(fInfoFrame, fTbtitle = new TGTextBuffer(100));
   fTbtitle->AddText(0, get_mode_name(wt->wavelet()));
   fTitle->Resize(300, fTitle->GetDefaultHeight());

   fInfoFrame->AddFrame(fLtitle, fL5);
   fInfoFrame->AddFrame(fTitle, fL6);

   AddFrame(fInfoFrame, fL1);

   // create radio buttons group
   fG1 = new TGGroupFrame(this, new TGString("Make wavelet transformation using..."));

   for (i = 0; i < 6; ++i) {
     fR[i] = new TGRadioButton(fG1,new TGHotString(get_mode_name(i+1)),11+i);
   }

   for (i = 0; i < 6; ++i) {
     fG1->AddFrame(fR[i], fL4);
     fR[i]->Associate(this);
   }

   int active=wt->wavelet();
   fR[active-1]->SetState(kButtonDown);

   AddFrame(fG1, fL3);

   // create radio buttons group
   fG2 = new TGGroupFrame(this, new TGString("Parameters for wavelet spectrum"));

   for(int i=0; i<3; i++)
     fParmFrame[i] = new TGCompositeFrame(fG2, 60, 20, kHorizontalFrame);

   char buff[100];
   sprintf(buff, "%s", "");  // to clear buff

   fT = new TGLabel(fParmFrame[0], new TGString("Title:"));
   fET = new TGTextEntry(fParmFrame[0], fBT = new TGTextBuffer(100));
   if(wt->is_fixed())sprintf(buff, "%s", wt->title());
   fBT->AddText(0, buff);
   fET->Resize(350, fET->GetDefaultHeight());
   fParmFrame[0]->AddFrame(fT, fL5);
   fParmFrame[0]->AddFrame(fET, fL6);

   fTy = new TGLabel(fParmFrame[1], new TGString("Scale-title:"));
   fETy = new TGTextEntry(fParmFrame[1], fBTy = new TGTextBuffer(100));
   if(wt->is_fixed())sprintf(buff, "%s", wt->ytitle());
   fBTy->AddText(0, buff);
   fETy->Resize(100, fETy->GetDefaultHeight());
   fParmFrame[1]->AddFrame(fTy, fL5);
   fParmFrame[1]->AddFrame(fETy, fL5);

   fYmax = new TGLabel(fParmFrame[2], new TGString("Scale-max:"));
   fEYmax = new TGTextEntry(fParmFrame[2], fBYmax = new TGTextBuffer(100));
   if(wt->is_fixed())sprintf(buff, "%f", wt->ymax());
   fBYmax->AddText(0, buff);
   fEYmax->Resize(64, fEYmax->GetDefaultHeight());
   fParmFrame[2]->AddFrame(fEYmax, fL6);
   fParmFrame[2]->AddFrame(fYmax, fL6);

   fYmin = new TGLabel(fParmFrame[2], new TGString("Scale-min:"));
   fEYmin = new TGTextEntry(fParmFrame[2], fBYmin = new TGTextBuffer(100));
   if(wt->is_fixed())sprintf(buff, "%f", wt->ymin());
   fBYmin->AddText(0, buff);
   fEYmin->Resize(64, fEYmin->GetDefaultHeight());
   fParmFrame[2]->AddFrame(fEYmin, fL6);
   fParmFrame[2]->AddFrame(fYmin, fL6);

   for(int i=0; i<3; i++)
     fG2->AddFrame(fParmFrame[i], fL1);

   fC=new TGCheckButton(fG2,new TGHotString("Use these parameters"),-1);
   fG2->AddFrame(fC, fL6);
   fC->Associate(this);
   if(wt->is_fixed())fC->SetState(kButtonDown);

   AddFrame(fG2, fL3);

   // Button frame
   fButtonFrame=new TGCompositeFrame(this, 60, 20, kHorizontalFrame | kFixedWidth);

   fOkButton = new TGTextButton(fButtonFrame, "&Ok", 1, fRedTextGC);
   fOkButton->SetToolTipText("Accept this representation");

   // Change background of fOkButton to green
   ULong_t green;
   fClient->GetColorByName("green", green);
   fOkButton->ChangeBackground(green);

   fCancelButton = new TGTextButton(fButtonFrame, "&Cancel", 2);
   fCancelButton->SetToolTipText("Close this window");

   fButtonFrame->Resize(GetDefaultWidth()/3, fButtonFrame->GetDefaultHeight());

   fOkButton->Associate(this);
   fCancelButton->Associate(this);

   fButtonFrame->AddFrame(fCancelButton, fL1);
   fButtonFrame->AddFrame(fOkButton, fL1);

   AddFrame(fButtonFrame, fL2);


   MapSubwindows();
   Resize(GetDefaultSize());

   // position relative to the parent's window
   Window_t wdum;
   gVirtualX->TranslateCoordinates(main->GetId(), GetParent()->GetId(),
                          (((TGFrame *) main)->GetWidth() - fWidth) >> 1,
                          (((TGFrame *) main)->GetHeight() - fHeight) >> 1,
                          ax, ay, wdum);
   Move(ax, ay);

   SetWindowName("Wavelet spectrum representation");

   MapWindow();
   fClient->WaitFor(this);
}

// Order is important when deleting frames. Delete children first,
// parents last.

GWTFrame::~GWTFrame()
{
   // Delete widgets created by dialog.

   delete fOkButton; delete fCancelButton;
   delete fR[0]; delete fR[1]; delete fR[2];
   delete fR[3]; delete fR[4]; delete fR[5];
   delete fTitle; delete fLtitle;
   delete fET; delete fETy; 
   delete fEYmin; delete fEYmax; 
   delete fT; delete fTy; 
   delete fYmin; delete fYmax; 
   delete fC;
   delete fParmFrame[0]; delete fParmFrame[1]; delete fParmFrame[2];
   delete fButtonFrame; delete fInfoFrame;
   delete fL1; delete fL2; delete fL3; delete fL4; delete fL5; delete fL6;
   gVirtualX->DeleteGC(fRedTextGC);
}

void GWTFrame::CloseWindow()
{
   // Close dialog in response to window manager close.

   delete this;
}

Bool_t GWTFrame::ProcessMessage(Long_t msg, Long_t parm1, Long_t)
{
  // Process messages sent to this dialog.

  int i;
  int old_wave=wt->wavelet();
  bool was_pressed=wt->is_fixed();

  switch(GET_MSG(msg))
    {
    case kC_COMMAND:

      switch(GET_SUBMSG(msg)) 
	{
	case kCM_BUTTON:
	  switch(parm1) 
	    {
	    case 1:  // Ok button
	      for(i=0; i<6; ++i)
		if(fR[i]->GetState() == kButtonDown)
		  *wt<<i+1;

	      wt->clear_parm();

	      if(fC->GetState() == kButtonDown) {
		wt->set_title(fBT->GetString());
		wt->set_ytitle(fBTy->GetString());
		wt->set_ymin(atof(fBYmin->GetString()));
		wt->set_ymax(atof(fBYmax->GetString()));
		wt->use_parm();
	      }

	      CloseWindow();
	      break;

	    case 2:  // Cancel button
	      CloseWindow();
	      break;

	    }
	  break;

	case kCM_RADIOBUTTON:
	case kCM_CHECKBUTTON:
	  if (parm1 >= 11 && parm1 <= 16)
	    {
	      for (i=0; i<6; ++i)
		if (fR[i]->WidgetId() != parm1)
		  fR[i]->SetState(kButtonUp);
	      fC->SetState(parm1-10 == old_wave && was_pressed 
			   ? kButtonDown : kButtonUp);
	    }
	  break;

	default:
	  break;
	}
      break;

    default:
      break;
    }

  return kTRUE;
}


char *GWTFrame::get_mode_name(int id)
{
  switch(id)
    {
    case 1: return "1 - first derivative of gauss";
    case 2: return "2 - second derivative of gauss (Mexican Hat)";
    case 3: return "3 - third derivative of gauss";
    case 4: return "4 - fourth derivative of gauss";
    case 5: return "5 - fifth derivative of gauss";
    case 6: return "6 - sixth derivative of gauss";
    }
  return "unknown";
}

