/***************************************************************************
    begin                : Wed Apr 10 2002
    copyright            : (C) 2002 by Alexei Soloviev
    email                : solovjev@lynx.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include<string>
#include<strstream>
#include<fstream>
#include<iomanip>

#include <TApplication.h>
#include <TGFileDialog.h>
#include <TCanvas.h>
#include <TGClient.h>
#include <TGString.h>

#include"MainFrame.h"
#include"DMFrame.h"
#include"GWTFrame.h"
#include"FilterFrame.h"
#include"Signal.h"


MainFrame::MainFrame(const TGWindow *p, UInt_t w, UInt_t h) : 
  TGMainFrame(p, w, h), 
  rod(), dm(), dv1("raw1","raw2"), dv2("ws1","ws2"), wt()
{
  // Create Read-Out Driver main frame. A TGMainFrame is a top level window.

  // Create menubar and popup menus. 
  fMenuBarLayout = new TGLayoutHints(kLHintsTop | kLHintsLeft | kLHintsExpandX,
                                      4, 4, 1, 1);
  fMenuBarItemLayout = new TGLayoutHints(kLHintsTop | kLHintsLeft, 0, 4, 0, 0);

  fMenuFile = new TGPopupMenu(fClient->GetRoot());
  fMenuFile->AddEntry("&Open...", M_FILE_OPEN);
  fMenuFile->AddEntry("&Reopen", M_FILE_REOPEN);
  fMenuFile->AddSeparator();
  fMenuFile->AddEntry("Save raw event image", M_FILE_SAVE_1);
  fMenuFile->AddEntry("Save wavelet spectrum image", M_FILE_SAVE_2);
  fMenuFile->AddSeparator();
  fMenuFile->AddEntry("Save wavelet spectrum matrix", M_FILE_DUMP);
  fMenuFile->AddSeparator();
  fMenuFile->AddEntry("E&xit", M_FILE_EXIT);

  fMenuOptions = new TGPopupMenu(fClient->GetRoot());
  fMenuOptions->AddEntry("Event &selection...", M_OPTION_FILTER);
  fMenuOptions->AddSeparator();
  fMenuOptions->AddEntry("&Event representation...", M_OPTION_PICTURE_1);
  fMenuOptions->AddEntry("&Wavelet spectrum representation...", M_OPTION_PICTURE_2);

  fMenuTools = new TGPopupMenu(fClient->GetRoot());
  fMenuTools->AddEntry("Next",M_TOOLS_NEXT);
  fMenuTools->AddEntry("Next 10",M_TOOLS_NEXT_10);
  fMenuTools->AddSeparator();
  fMenuTools->AddEntry("Prev",M_TOOLS_PREV);
  fMenuTools->AddEntry("Prev 10",M_TOOLS_PREV_10);
  fMenuTools->AddSeparator();
  fMenuTools->AddEntry("All",M_TOOLS_ALL);

  fMenuFile->Associate(this);
  fMenuOptions->Associate(this);
  fMenuTools->Associate(this);

  fMenuBar = new TGMenuBar(this, 1, 1, kHorizontalFrame);
  fMenuBar->AddPopup("&File", fMenuFile, fMenuBarItemLayout);
  fMenuBar->AddPopup("&Options", fMenuOptions, fMenuBarItemLayout);
  fMenuBar->AddPopup("&Tools", fMenuTools, fMenuBarItemLayout);

  AddFrame(fMenuBar, fMenuBarLayout);

  // Create info frame
  fInfoFrame=new TGCompositeFrame(this, 60, 20, kVerticalFrame | kSunkenFrame);
  fInfoLayout=new TGLayoutHints(kLHintsTop | kLHintsLeft | kLHintsExpandX, 
				4, 4, 1, 1);

  fInfoFile  = new TGLabel(fInfoFrame, "");  // "File: ..."
  fInfoEvent = new TGLabel(fInfoFrame, "");  // "Event no. ... (multiplicity: ...)"

  fInfoFrame->AddFrame(fInfoFile, fInfoLayout);
  fInfoFrame->AddFrame(fInfoEvent, fInfoLayout);

  AddFrame(fInfoFrame, fInfoLayout);

  // Create work frame
  fWorkFrame = new TGCompositeFrame(this, 60, 60, kHorizontalFrame);

  fWorkFrameLayout = new TGLayoutHints(kLHintsTop | kLHintsLeft |
				       kLHintsExpandX | kLHintsExpandY,
				       2, 2, 2, 2);

  fWFLayout = new TGLayoutHints(kLHintsTop | kLHintsLeft |
				kLHintsExpandX, 2, 2, 2, 2);

  // Create extracter work frame (fWorkFrame_1)
  fWorkFrame_1 = new TGGroupFrame(fWorkFrame, "Raw event");

  const int pad_w1=300;
  const int pad_h1=300;

  fEc1 = new TRootEmbeddedCanvas("ec1", fWorkFrame_1, pad_w1, pad_h1);
  fWorkFrame_1->AddFrame(fEc1, fWorkFrameLayout);
  fEc1->GetCanvas()->SetBorderMode(0);
  fEc1->GetCanvas()->SetFillColor(10);
  dv1.associate(fEc1->GetCanvas());

  fOptBar_1 = new TGCompositeFrame(fWorkFrame_1, 60, 20, kHorizontalFrame);
  fWorkBar_1 = new TGCompositeFrame(fWorkFrame_1, 60, 20, kHorizontalFrame);

  fPictureButton_1 = new TGTextButton(fOptBar_1, "Picture", 1101);
  fPictureButton_1->Associate(this);
  fPictureButton_1->SetToolTipText("Choose event representation manner");
  fOptBar_1->AddFrame(fPictureButton_1, fWFLayout);

  fSaveButton_1 = new TGTextButton(fOptBar_1, "Save", 1102);
  fSaveButton_1->Associate(this);
  fSaveButton_1->SetToolTipText("Save raw event image to file");
  fOptBar_1->AddFrame(fSaveButton_1, fWFLayout);

  fPrev10Button_1 = new TGTextButton(fWorkBar_1, "Prev 10", 1210);
  fPrev10Button_1->Associate(this);
  fPrev10Button_1->SetToolTipText("Get previous event, omitting 10 ones");
  fWorkBar_1->AddFrame(fPrev10Button_1, fWFLayout);

  fPrevButton_1 = new TGTextButton(fWorkBar_1, "Prev", 1200);
  fPrevButton_1->Associate(this);
  fPrevButton_1->SetToolTipText("Get previous event");
  fWorkBar_1->AddFrame(fPrevButton_1, fWFLayout);

  fNextButton_1 = new TGTextButton(fWorkBar_1, "Next", 1201);
  fNextButton_1->Associate(this);
  fNextButton_1->SetToolTipText("Get next event");
  fWorkBar_1->AddFrame(fNextButton_1, fWFLayout);

  fNext10Button_1 = new TGTextButton(fWorkBar_1, "Next 10", 1211);
  fNext10Button_1->Associate(this);
  fNext10Button_1->SetToolTipText("Get next event, omitting 10 ones");
  fWorkBar_1->AddFrame(fNext10Button_1, fWFLayout);

  fWorkFrame_1->AddFrame(fOptBar_1, fWFLayout);
  fWorkFrame_1->AddFrame(fWorkBar_1, fWFLayout);

  // Create analyser work frame (fWorkFrame_2)
  fWorkFrame_2 = new TGGroupFrame(fWorkFrame, "Wavelet spectrum");

  const int pad_w2=300;
  const int pad_h2=300;

  fEc2 = new TRootEmbeddedCanvas("ec2", fWorkFrame_2, pad_w2, pad_h2);
  fWorkFrame_2->AddFrame(fEc2, fWorkFrameLayout);
  fEc2->GetCanvas()->SetBorderMode(0);
  fEc2->GetCanvas()->SetFillColor(10);
  dv2.associate(fEc2->GetCanvas());

  fOptBar_2 = new TGCompositeFrame(fWorkFrame_2, 60, 20, kHorizontalFrame);
  fWorkBar_2 = new TGCompositeFrame(fWorkFrame_2, 60, 20, kHorizontalFrame);

  fPictureButton_2 = new TGTextButton(fOptBar_2, "Picture", 2101);
  fPictureButton_2->Associate(this);
  fPictureButton_2->SetToolTipText("Choose wavelet spectrum representation manner");
  fOptBar_2->AddFrame(fPictureButton_2, fWFLayout);

  fSaveButton_2 = new TGTextButton(fOptBar_2, "Save", 2102);
  fSaveButton_2->Associate(this);
  fSaveButton_2->SetToolTipText("Save wavelet spectrum image to file");
  fOptBar_2->AddFrame(fSaveButton_2, fWFLayout);

  fDumpButton_2 = new TGTextButton(fOptBar_2, "Dump", 2103);
  fDumpButton_2->Associate(this);
  fDumpButton_2->SetToolTipText("Save wavelet spectrum matrix to file");
  fOptBar_2->AddFrame(fDumpButton_2, fWFLayout);




  fScale_2 = new TGTextEntry(fWorkBar_2, 
			     fBScale_2 = new TGTextBuffer(10), SCALE);
  fScale_2->Resize(40,fScale_2->GetDefaultHeight());
  fBScale_2->AddText(0, "0");
  fScale_2->Associate(this);
  fWorkBar_2->AddFrame(fScale_2, fWFLayout);

  fSlider_2 = new TGHSlider(fWorkBar_2, 110, kSlider1 | kScaleBoth, S_SCALE);
  fSlider_2->Associate(this);
  fSlider_2->SetRange(0,100);
  fSlider_2->SetPosition(0);
  fWorkBar_2->AddFrame(fSlider_2, fWFLayout);

  fShowButton_2 = new TGTextButton(fWorkBar_2, "Slice/Filt", 2202);
  fShowButton_2->Associate(this);
  fShowButton_2->SetToolTipText("Draw slice of wavelet spectrum (1D) or filtered image (2D)");
  fWorkBar_2->AddFrame(fShowButton_2, fWFLayout);


  fDrawButton_2 = new TGTextButton(fWorkBar_2, "Wavelet", 2201);
  fDrawButton_2->Associate(this);
  fDrawButton_2->SetToolTipText("Draw/Redraw wavelet spectrum");
  fWorkBar_2->AddFrame(fDrawButton_2, fWFLayout);

  fEnergyButton_2 = new TGTextButton(fWorkBar_2, "Energy", 2203);
  fEnergyButton_2->Associate(this);
  fEnergyButton_2->SetToolTipText("Draw energy wavelet spectrum");
  fWorkBar_2->AddFrame(fEnergyButton_2, fWFLayout);

  fEWAButton_2 = new TGTextButton(fWorkBar_2, "Scalogr", 2204);
  fEWAButton_2->Associate(this);
  fEWAButton_2->SetToolTipText("Integrate energy wavelet spectrum");
  fWorkBar_2->AddFrame(fEWAButton_2, fWFLayout);


  fWorkFrame_2->AddFrame(fOptBar_2, fWFLayout);
  fWorkFrame_2->AddFrame(fWorkBar_2, fWFLayout);


  fWorkFrame->AddFrame(fWorkFrame_1, fWorkFrameLayout);
  fWorkFrame->AddFrame(fWorkFrame_2, fWorkFrameLayout);

  AddFrame(fWorkFrame, fWorkFrameLayout);

  // Create status frame containing text entriy widgets
  fStatusFrame = new TGCompositeFrame(this, 60, 20, kHorizontalFrame | kSunkenFrame);

  fWarning = new TGLabel(fStatusFrame, "");
  fStatusFrame->AddFrame(fWarning, new TGLayoutHints(kLHintsBottom | kLHintsLeft, 
						     4, 2, 2, 2));
  AddFrame(fStatusFrame, fWFLayout);



  SetWindowName("Wavelet Analysis of Secondary Particles angular distributions");

  MapSubwindows();

  // we need to use GetDefault...() to initialize the layout algorithm...
  Resize(GetDefaultSize());
  //Resize(400, 200);

  MapWindow();
}

MainFrame::~MainFrame()
{
  // Delete all created widgets.

  delete fMenuBar;
  delete fMenuFile; delete fMenuOptions; delete fMenuTools;
  delete fMenuBarLayout; delete fMenuBarItemLayout;

  delete fInfoFrame;
  delete fInfoFile; delete fInfoEvent;
  delete fInfoLayout;

  delete fWorkFrame;

  delete fWorkFrame_1;
  delete fEc1;
  delete fOptBar_1; delete fWorkBar_1;
  delete fPictureButton_1; delete fSaveButton_1;
  delete fPrevButton_1; delete fNextButton_1;
  delete fPrev10Button_1; delete fNext10Button_1;

  delete fWorkFrame_2;
  delete fEc2;
  delete fOptBar_2; delete fWorkBar_2;
  delete fPictureButton_2; delete fSaveButton_2;
  delete fDrawButton_2;
  delete fEnergyButton_2;
  delete fEWAButton_2;

   delete fSlider_2;
   delete fScale_2;
  delete fShowButton_2;
  delete fDumpButton_2;

  delete fWorkFrameLayout; delete fWFLayout;

  delete fStatusFrame;
  delete fWarning;
}

void MainFrame::CloseWindow()
{
  // Got close message for this MainFrame. Calls parent CloseWindow()
  // (which destroys the window) and terminate the application.
  // The close message is generated by the window manager when its close
  // window menu item is selected.

  TGMainFrame::CloseWindow();
  gApplication->Terminate(0);
}

Bool_t MainFrame::ProcessMessage(Long_t msg, Long_t parm1, Long_t parm2)
{
  // Handle messages send to the TestMainFrame object. E.g. all menu button
  // messages.

  const char *filetypes[] = { "All files",     "*",
			      "Data files",    "*.dat",
			      "ROOT files",    "*.root",
			      "ROOT macros",   "*.C",
			      0,               0 };

  switch(GET_MSG(msg))
    {
    case kC_COMMAND:
      switch(GET_SUBMSG(msg))
	{
	case kCM_BUTTON:
	  if(parm1 == 1210 || parm1 == 1200 || parm1 == 1201 || parm1 == 1211) 
	    {
	      try{
		if(parm1 == 1210) rod.prev(10);  // Prev10 button
		if(parm1 == 1200) rod.prev();  // Prev button
		if(parm1 == 1201) rod.next();  // Next button
		if(parm1 == 1211) rod.next(10);  // Next10 button
		dm<<rod.current();
	      }
	      catch(Read_Out_Error &e){
		warn(e.warning());
		break;
	      }
	      warn();

	      draw_1();
	      draw_2();

	      char buff[100];
	      sprintf(buff, "%d", rod.count());
	      std::string event_info=buff;
	      event_info+="  (multiplicity: ";
	      sprintf(buff, "%d", rod.current().size());
	      event_info+=buff;
	      event_info+=")";

	      info_event(event_info);
	    }

	  if(parm1 == 1101 || parm1 == 2101)  // Picture buttons
	    {
	      if(parm1 == 1101)
		new DMFrame(&dm, fClient->GetRoot(), this, 400, 200);
	      if(parm1 == 2101)
		new GWTFrame(&wt, fClient->GetRoot(), this, 400, 200);

	      try{ 
		dm<<rod.current();
	      }
	      catch(Read_Out_Error &e){ break; }

	      if(parm1 == 1101) draw_1();
	      draw_2();
	    }

	  if(parm1 == 1102 || parm1 == 2102)  // Save buttons
	    {
	      if(!rod.count()) {
		if(parm1 == 1102) warn("No raw event image to be saved");
		if(parm1 == 2102) warn("No wavelet spectrum image to be saved");
		break;
	      }

	      char buff[100];
	      std::string file;
	      if(parm1 == 1102) file="e";
	      if(parm1 == 2102) file="ws";
	      sprintf(buff, "%d", rod.count());
	      file+=buff;
	      file+="_";
	      sprintf(buff, "%d", dm.state());
	      file+=buff;
	      file+=".eps";
	      if(parm1 == 1102) fEc1->GetCanvas()->SaveAs(file.c_str());
	      if(parm1 == 2102) fEc2->GetCanvas()->SaveAs(file.c_str());
	      std::string mesage="File \""+file+"\" has been created";
	      warn(mesage);
	    }

	  if(parm1 == 2201)  // Wavelet button
	    {
	      try{
		dm<<rod.current();
	      }
	      catch(Read_Out_Error &e){
		warn(e.warning());
		break;
	      }
	      warn();

	      draw_2(128);
	    }

	  if(parm1 == 2203)  // Energy button
	    {
	      try{
		dm<<rod.current();
	      }
	      catch(Read_Out_Error &e){
		warn(e.warning());
		break;
	      }
	      warn();

	      (dv2<<"col").use_palette();

	      H2 es;

	      if(dm.state() < 3) {
		H1 pattern=dm.hist_1(128);
//  		es=wt.transform(pattern);
		es=wt.transform(dm.sample(), &pattern);
	      } else if(dm.state() == 3) {
		H1 pattern=dm.hist_1(128);
		Sample s=dm.sample();
		Sample aux=s;
		for(Sample::const_iterator p=s.begin(); p != s.end(); p++) {
		  aux.push_back(*p+2*3.1415926536);
		  aux.push_back(*p-2*3.1415926536);
		  aux.push_back(*p+4*3.1415926536);
		  aux.push_back(*p-4*3.1415926536);
		}
		es=wt.transform(aux, &pattern);
	      } else {
		double scale=0.01+atof(fBScale_2->GetString())/100;
		es=wt.transform(dm.hist_2(128),scale);
	      }

	      for(int i=0; i<es.ybins(); i++)
		for(int j=0; j<es.xbins(); j++)
		  es[i][j]*=es[i][j];

	      dv2<<es<<"";
	    }

	  if(parm1 == 2204)  // Scalogram button
	    {
	      try{
		dm<<rod.current();
	      }
	      catch(Read_Out_Error &e){
		warn(e.warning());
		break;
	      }
	      warn();

	      if(dm.state() < 3) {
		Sample sample = dm.sample();
		H1 pattern = dm.hist_1(128);
//  		H2 ws = wt.transform(pattern);
		H2 ws = wt.transform(sample, &pattern);
		H1 scalo(ws.ybins(), ws.ymin(), ws.ymax(), " Scalogram");
		double length = ws.xmax() - ws.xmin();
//  		for(int i=0; i<ws.ybins(); i++)
//  		  for(double x=ws.xmin()-length; x<ws.xmax()+length; x+=ws.xstep())
//  		    scalo[i] += pow(wt.transform(sample, ws.i2y(i), x),2) * ws.xstep();
		for(int i=0; i<ws.ybins(); i++) {
		  for(int j=0; j<ws.xbins(); j++)
		    scalo[i] += pow(ws[i][j],2) * ws.xstep();
		  for(double x=ws.xmin()-length; x<ws.xmin(); x+=ws.xstep())
		    scalo[i] += pow(wt.transform(sample, ws.i2y(i), x),2) * ws.xstep();
		  for(double x=ws.xmax(); x<ws.xmax()+length; x+=ws.xstep())
		    scalo[i] += pow(wt.transform(sample, ws.i2y(i), x),2) * ws.xstep();
		}

		dv2<<""<<scalo;
	      } else if(dm.state() == 3) {
		Sample s = dm.sample();
		Sample aux = s;
		for(Sample::const_iterator p=s.begin(); p != s.end(); p++) {
		  aux.push_back(*p+2*3.1415926536);
		  aux.push_back(*p-2*3.1415926536);
		  aux.push_back(*p+4*3.1415926536);
		  aux.push_back(*p-4*3.1415926536);
		}
		H1 pattern = dm.hist_1(128);
		H2 ws = wt.transform(aux, &pattern);
		H1 scalo(ws.ybins(), ws.ymin(), ws.ymax(), " Scalogram");
		for(int i=0; i<ws.ybins(); i++) {
		  for(int j=0; j<ws.xbins(); j++)
		    scalo[i] += pow(ws[i][j],2) * ws.xstep();
		}

		dv2<<""<<scalo;
	      } else {
		H1 int_es(64, 0.01, 1.01, " Scalogram");
		H2 es;
		for(int s=0; s<int_es.xbins(); s++) {
		  es=wt.transform(dm.hist_2(int_es.xbins()), int_es.i2x(s));
		  for(int i=0; i<es.ybins(); i++)
		    for(int j=0; j<es.xbins(); j++) {
		      es[i][j] *= es[i][j];
		      int_es[s] += es[i][j] * es.xstep() * es.ystep();
		    }
		}
		dv2<<""<<int_es;
	      }
	    }

	  if(parm1 == 2202)  // Slice / Filt button
	    {

	      try{
		dm<<rod.current();
	      }
	      catch(Read_Out_Error &e){
		warn(e.warning());
		break;
	      }

	      if(dm.state() < 3) {
		H1 pattern=dm.hist_1(128);
		Sample sample=dm.sample();
		double scale=wt.transform(pattern).ymax()*
		  atof(fBScale_2->GetString())/100;
		dv2<<""<<wt.transform(sample,scale,&pattern);

		char buff[100];
		sprintf(buff, "%f", scale);
		warn(std::string("Wavelet spectrum slice at scale ")+
		     buff+" is displayed");
	      } else if(dm.state() == 3) {
		H1 pattern=dm.hist_1(128);
		Sample s=dm.sample();
		Sample aux=s;
		for(Sample::const_iterator p=s.begin(); p != s.end(); p++) {
		  aux.push_back(*p+2*3.1415926536);
		  aux.push_back(*p-2*3.1415926536);
		  aux.push_back(*p+4*3.1415926536);
		  aux.push_back(*p-4*3.1415926536);
		}
		double scale=wt.transform(pattern).ymax()*
		  atof(fBScale_2->GetString())/100;
		dv2<<""<<wt.transform(aux,scale,&pattern);


		char buff[100];
		sprintf(buff, "%f", scale);
		warn(std::string("Wavelet spectrum slice at scale ")+
		     buff+" is displayed");
	      } else {
		(dv2<<"col").use_palette();
		int scale=static_cast<int>(5*atof(fBScale_2->GetString())/100);
		dv2<<wt.transform(dm.hist_2(256), 5, scale); 

		char buff[100];
		sprintf(buff, "%d", scale);
		warn(std::string("Filtered image is displayed: ")+
		     buff+" of 5 layers are set in zero");
	      }
	    }

	  if(parm1 == 2103)  // Dump button
	    {
	      if(!rod.count())
		{
		  warn("No wavelet spectrum matrix to be saved");
		  break;
		}

	      char buff[100];
	      std::string file="wcm";
	      sprintf(buff, "%d", rod.count());
	      file+=buff;
	      file+="_";
	      sprintf(buff, "%d", dm.state());
	      file+=buff;
	      file+=".dat";
	      std::string mesage="File \""+file+"\" has been created";
	      warn(mesage);

	      H2 m;
	      if(dm.state() < 3) {
		H1 pattern=dm.hist_1(64);
		m=wt.transform(dm.sample(), &pattern);
	      } else if(dm.state() == 3) {
		Sample s=dm.sample();
		Sample aux=s;
		for(Sample::const_iterator p=s.begin(); p != s.end(); p++) {
		  aux.push_back(*p+2*3.1415926536);
		  aux.push_back(*p-2*3.1415926536);
		  aux.push_back(*p+4*3.1415926536);
		  aux.push_back(*p-4*3.1415926536);
		}
		H1 pattern=dm.hist_1(64);
		m=wt.transform(aux, &pattern);
	      } else {
		double scale=0.01+atof(fBScale_2->GetString())/100;
		m=wt.transform(dm.hist_2(64), scale); 
	      }

	      
	      std::ofstream f(file.c_str(),ios::out);
	      const int n_in_line=6;
	      int i_in_line=1;

	      f.setf(ios::scientific,ios::floatfield);
	      f.setf(ios::right,ios::adjustfield);
	      f.precision(4);

	      if(f.is_open()) {
		for(int i=0; i<m.xbins(); i++, ++i_in_line) {
		  f<<setw(12)<<m.i2x(i);
		  if(i_in_line>=n_in_line) {
		    f<<endl;
		    i_in_line=0;
		  }
		}
		f<<endl<<endl;
		i_in_line=1;
		for(int i=0; i<m.ybins(); i++, ++i_in_line) {
		  f<<setw(12)<<m.i2y(i);
		  if(i_in_line>=n_in_line) {
		    f<<endl;
		    i_in_line=0;
		  }
		}
		f<<endl<<endl;
		i_in_line=1;
		for(int i=0; i<m.ybins(); i++) {
		  for(int j=0; j<m.xbins(); j++, ++i_in_line) {
		    f<<setw(12)<<m[i][j];
		    if(i_in_line>=n_in_line) {
		    f<<endl;
		    i_in_line=0;
		    }
		  }
		  f<<endl;
		  i_in_line=1;
		}
	      }
	      f.close();
	    }
	  break;

	case kCM_MENU:
	  switch(parm1)
	    {
	    case M_FILE_OPEN: {
	      TGFileInfo fi;
	      fi.fFileTypes = filetypes;
	      new TGFileDialog(fClient->GetRoot(), this, kFDOpen, &fi);
	      if(!fi.fFilename)break;
	      try{
		rod.open_file(fi.fFilename);
	      }
	      catch(Read_Out_Error &e){
		warn(e.warning());
		info_file();
		info_event();
		break;
	      }
	      warn();
	      info_file(rod.name());
	      info_event();
	    }
	      break;

	    case M_FILE_REOPEN:
	      try{
		rod.reopen_file();
	      }
	      catch(Read_Out_Error &e){
		warn(e.warning());
		info_file();
		info_event();
		break;
	      }
	      warn();
	      info_file(rod.name());
	      info_event();
	      break;

	    case M_FILE_SAVE_1:
	    case M_FILE_SAVE_2:
	      {
		if(!rod.count()) {
		  if(parm1 == M_FILE_SAVE_1)
		    warn("No raw event image to be saved");
		  if(parm1 == M_FILE_SAVE_2)
		    warn("No wavelet spectrum image to be saved");
		  break;
		}

		char buff[100];
		std::string file;
		if(parm1 == M_FILE_SAVE_1) file="e";
		if(parm1 == M_FILE_SAVE_2) file="ws";
		sprintf(buff, "%d", rod.count());
		file+=buff;
		file+="_";
		sprintf(buff, "%d", dm.state());
		file+=buff;
		file+=".eps";
		if(parm1 == M_FILE_SAVE_1) fEc1->GetCanvas()->SaveAs(file.c_str());
		if(parm1 == M_FILE_SAVE_2) fEc2->GetCanvas()->SaveAs(file.c_str());
		std::string mesage="File \""+file+"\" has been created";
		warn(mesage);
	      }
	      break;

	    case M_FILE_DUMP:
	      {
		if(!rod.count()) {
		  warn("No wavelet spectrum matrix to be saved");
		  break;
		}

		char buff[100];
		std::string file="wcm";
		sprintf(buff, "%d", rod.count());
		file+=buff;
		file+="_";
		sprintf(buff, "%d", dm.state());
		file+=buff;
		file+=".dat";
		std::string mesage="File \""+file+"\" has been created";
		warn(mesage);

		H2 m;
		if(dm.state() <= 3) {
		  H1 pattern=dm.hist_1(64);
		  m=wt.transform(pattern);
		}

		std::ofstream f(file.c_str(),ios::out);
		const int n_in_line=6;
		int i_in_line=1;

		f.setf(ios::scientific,ios::floatfield);
		f.setf(ios::right,ios::adjustfield);
		f.precision(4);

		if(f.is_open()) {
		  for(int i=0; i<m.xbins(); i++, ++i_in_line) {
		    f<<setw(12)<<m.i2x(i);
		    if(i_in_line>=n_in_line) {
		      f<<endl;
		      i_in_line=0;
		    }
		  }
		  f<<endl<<endl;
		  i_in_line=1;
		  for(int i=0; i<m.ybins(); i++, ++i_in_line) {
		    f<<setw(12)<<m.i2y(i);
		    if(i_in_line>=n_in_line) {
		      f<<endl;
		      i_in_line=0;
		    }
		  }
		  f<<endl<<endl;
		  i_in_line=1;
		  for(int i=0; i<m.ybins(); i++) {
		    for(int j=0; j<m.xbins(); j++, ++i_in_line) {
		      f<<setw(12)<<m[i][j];
		      if(i_in_line>=n_in_line) {
			f<<endl;
			i_in_line=0;
		      }
		    }
		    f<<endl;
		    i_in_line=1;
		  }
		}
		f.close();
	      }
	      break;

	    case M_FILE_EXIT:
	      CloseWindow();  // this also terminates theApp
	      break;

	    case M_OPTION_FILTER:
	      new FilterFrame(rod.filter(), fClient->GetRoot(), this, 400, 200);
	      break;

	    case M_OPTION_PICTURE_1:
	    case M_OPTION_PICTURE_2:
	      {
		if(parm1 == M_OPTION_PICTURE_1)
		  new DMFrame(&dm, fClient->GetRoot(), this, 400, 200);
		if(parm1 == M_OPTION_PICTURE_2)
		  new GWTFrame(&wt, fClient->GetRoot(), this, 400, 200);

		try{ 
		  dm<<rod.current();
		}
		catch(Read_Out_Error &e){ break; }

		if(parm1 == M_OPTION_PICTURE_1) draw_1();
		draw_2();
	      }
	      break;

	    case M_TOOLS_NEXT:
	    case M_TOOLS_NEXT_10:
	    case M_TOOLS_PREV:
	    case M_TOOLS_PREV_10:
	      {
		try{
		  if(parm1 == M_TOOLS_NEXT)rod.next();
		  if(parm1 == M_TOOLS_NEXT_10)rod.next(10);
		  if(parm1 == M_TOOLS_PREV)rod.prev();
		  if(parm1 == M_TOOLS_PREV_10)rod.prev(10);
		  dm<<rod.current();
		}
		catch(Read_Out_Error &e){
		  warn(e.warning());
		  break;
		}
		warn();

		draw_1();
		draw_2();

		char buff[100];
		sprintf(buff, "%d", rod.count());
		std::string event_info=buff;
		event_info+="  (multiplicity: ";
		sprintf(buff, "%d", rod.current().size());
		event_info+=buff;
		event_info+=")";

		info_event(event_info);
	      }
	      break;

	    case M_TOOLS_ALL:
	      {
		warn("Working...");
		try{
		  rod.all();
		  dm<<rod.current();
		}
		catch(Read_Out_Error &e){
		  warn(e.warning());
		  break;
		}
		warn("Done!");

		draw_1();
		draw_2();

		std::string event_info="collection of ";
		char buff[100];
		sprintf(buff, "%d", rod.count_selected());
		event_info+=buff;
		event_info+=" / ";
		sprintf(buff, "%d", rod.count());
		event_info+=buff;
		event_info+=" events";
		info_event(event_info);
	      }
	      break;

	    default:
	      break;
	    }
	default:
	  break;
	}

  case kC_TEXTENTRY:
    switch (GET_SUBMSG(msg))
      {
      case kTE_TEXTCHANGED:
	switch (parm1)
	  {
	  case SCALE:
	    fSlider_2->SetPosition(atoi(fBScale_2->GetString()));
	    break;
	  }
	break;
      }
    break;

  case kC_HSLIDER:
    switch (GET_SUBMSG(msg))
      {
      case kSL_POS:
	char buff[10];
	sprintf(buff, "%ld", parm2);
	switch (parm1)
	  {
	  case S_SCALE:
	    sprintf(buff, "%d", fSlider_2->GetPosition());
	    fBScale_2->Clear();
	    fBScale_2->AddText(0, buff);
	    fClient->NeedRedraw(fScale_2);
	    break;
	  }
      }
    break;

    default:
      break;
    }
  return kTRUE;
}


void MainFrame::draw_1()
{
    switch(dm.state()) {
    case 1: case 2: case 3: dv1<<dm.hist_1(); break;
    case 4: case 5: case 6: dv1<<dm.hist_2(); break;
    }
}

void MainFrame::draw_2(int sharp=64)
{
  (dv2<<"col").use_palette();

  H1 pattern=dm.hist_1(sharp);

  switch(dm.state()) {
  case 1: case 2: 
//      dv2<<wt.transform(pattern); 
    dv2<<wt.transform(dm.sample(), &pattern); 
    break;
  case 3: {
    Sample s=dm.sample();
    Sample aux=s;
    for(Sample::const_iterator p=s.begin(); p != s.end(); p++) {
      aux.push_back(*p+2*3.1415926536);
      aux.push_back(*p-2*3.1415926536);
      aux.push_back(*p+4*3.1415926536);
      aux.push_back(*p-4*3.1415926536);
    }
    dv2<<wt.transform(aux, &pattern); 
  }
  break;
  case 4: case 5: case 6: 
    double scale=0.01+atof(fBScale_2->GetString())/100;
    dv2<<wt.transform(dm.hist_2(sharp), scale); 
    break;
  }
  dv2<<"";
}

void MainFrame::warn(std::string msg="")
{
  fWarning->SetText(new TGString(msg.c_str()));
  fWarning->Resize(fWarning->GetDefaultSize());
}

void MainFrame::info_file(std::string msg="")
{
  msg = msg.size() ? std::string("File: ")+msg : msg;
  fInfoFile->SetText(new TGString(msg.c_str()));
  fInfoFile->Resize(fInfoFile->GetDefaultSize());
}

void MainFrame::info_event(std::string msg="")
{
  msg = msg.size() ? std::string("Event: ")+msg : msg;
  fInfoEvent->SetText(new TGString(msg.c_str()));
  fInfoEvent->Resize(fInfoEvent->GetDefaultSize());
}
