/***************************************************************************
    begin                : Wed Apr 10 2002
    copyright            : (C) 2002 by Alexei Soloviev
    email                : solovjev@lynx.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include<algorithm>
#include<cmath>

#include"ROD.h"


ROD::ROD() : 
  input(0), input_name(), 
  current_event(), ROB(), curr_pos(0), event_counter(0), selected_event_counter(0), 
  own_filter() 
{}

ROD::ROD(std::istream &i, const char *n="<anonymous input>") : 
  input(&i), input_name(n), 
  current_event(), ROB(), curr_pos(0), event_counter(0), selected_event_counter(0), 
  own_filter() 
{}

ROD::ROD(std::istream *i, const char *n="<anonymous input>") : 
  input(i), input_name(n), 
  current_event(), ROB(), curr_pos(0), event_counter(0), selected_event_counter(0), 
  own_filter() 
{}

void ROD::open_input(std::istream &i, const char *n="<anonymous input>")
{
  input=&i;
  input_name=n;
  current_event.clear();
  ROB.clear();
  curr_pos=0;
  event_counter=0;
  selected_event_counter=0;
}

void ROD::open_input(std::istream *i, const char *n="<anonymous input>")
{
  input=i;
  input_name=n;
  current_event.clear();
  ROB.clear();
  curr_pos=0;
  event_counter=0;
  selected_event_counter=0;
}

void ROD::open_file(const char *n)
{
  std::ifstream *file=dynamic_cast<std::ifstream *>(input);

  if(file)file->close();
  else file=new std::ifstream;

  file->open(n);

  if(file->is_open())open_input(file,n);
  else throw No_File();
}

void ROD::reopen_file()
{
  open_file(input_name.c_str());
}

const Event &ROD::current() const 
{
  if(!input)throw No_File();
  if(current_event.empty())throw No_Event();
  return current_event;
}

void ROD::next(int step=1)
{
  if(!input)throw No_File();
  if(input->eof())throw End_of_File();

  for(int i=0; i<step; i++) {
    if(ROB.empty() || curr_pos == --ROB.end()) {
      *input>>current_event;
      ROB.push_back(current_event);
      curr_pos=--ROB.end();
    }
    else current_event = *(++curr_pos);
    ++event_counter;

    if(!own_filter.test_event(current_event))return next();
    ++selected_event_counter;

    if(input->eof())return;
  }
}

void ROD::prev(int step=1)
{
  if(!input)throw No_File();
  if(ROB.empty() || curr_pos == ROB.begin())throw No_Event();

  for(int i=0; i<step; i++) {
    current_event = *(--curr_pos);
    --event_counter;

    if(!own_filter.test_event(current_event))return prev();
    --selected_event_counter;

    if(curr_pos == ROB.begin())return;
  }
}

void ROD::all()
{
  if(!input)throw No_File();
  if(input->eof())throw End_of_File();

  Event event;

  while(!input->eof()) {
    if(ROB.empty() || curr_pos == --ROB.end()) {
      *input>>event;
      ROB.push_back(event);
      curr_pos=--ROB.end();
    }
    else event = *(++curr_pos);
    ++event_counter;

    if(own_filter.test_event(event)) {
      current_event+=event;
      ++selected_event_counter;
    }
  }
}

