/***************************************************************************
    begin                : Wed Apr 10 2002
    copyright            : (C) 2002 by Alexei Soloviev
    email                : solovjev@lynx.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LRCD_ROD_H
#define LRCD_ROD_H

#include<iostream>
#include<fstream>
#include<list>
#include<string>

#include"Event.h"
#include"Filter.h"


class ROD  // Read-Out Driver
{
  std::istream *input;
  std::string input_name;

  Event current_event;

  std::list<Event> ROB;  // Read-Out Buffer
  std::list<Event>::const_iterator curr_pos;

  int event_counter;
  int selected_event_counter;

  Filter own_filter;

 public:
  ROD();
  ROD(std::istream &i, const char *n="<anonymous input>");
  ROD(std::istream *i, const char *n="<anonymous input>");
  ~ROD(){}

  void open_input(std::istream &i, const char *n="<anonymous input>");
  void open_input(std::istream *i, const char *n="<anonymous input>");
  void open_file(const char *n);
  void reopen_file();

  inline Filter *filter() { return &own_filter; }

  const Event &current() const;  // get reference to current event

  void next(int step=1);  // get next event from ROB or input
  void prev(int step=1);  // get prev event from ROB
  void all();  // get all events from input

  inline std::string name() const { return input_name; }

  inline int count() const { return event_counter; }
  inline int count_selected() const { return selected_event_counter; }
};

// Exeptions:

class Read_Out_Error{
 public:
  virtual std::string warning() const = 0;
};

class No_File: public Read_Out_Error{
 public:
  virtual std::string warning() const { return "No file"; }
};

class End_of_File: public Read_Out_Error{
 public:
  virtual std::string warning() const { return "End of file"; }
};

class No_Event: public Read_Out_Error{
 public:
  virtual std::string warning() const { return "No event"; }
};

#endif  // LRCD_ROD_H
