/***************************************************************************
    begin                : Wed Apr 10 2002
    copyright            : (C) 2002 by Alexei Soloviev
    email                : solovjev@lynx.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include"Signal.h"


H1::H1()
  : Vector(), x_min(0), x_max(0), name(), x_name(), y_name() 
{}

H1::H1(int nbinsx, const double & xmin, const double & xmax, const char *title="",
     const char *xtitle="", const char *ytitle="")
  : Vector(nbinsx), x_min(xmin), x_max(xmax), name(title), 
  x_name(xtitle), y_name(ytitle) 
{}

int H1::fill(double x, double w=1)
{
  unsigned int i=x2i(x);
  if( i>=0 && i<size() ) operator[](i)+=w;
  return i;
}

double H1::at(double x) const
{
  unsigned int i=x2i(x);
  unsigned int n=size();

  if( i<0 ) return operator[](i+n);  // signal is periodic
  if( i>=n ) return operator[](i-n);  // signal is periodic
  return operator[](i);
}


H2::H2() : 
  Matrix(), 
  x_bins(0), x_min(0), x_max(0), 
  y_bins(0), y_min(0), y_max(0), 
  name(), 
  x_name(), y_name() 
{}

H2::H2(int nbinsx, const double & xmin, const double & xmax,
       int nbinsy, const double & ymin, const double & ymax,
       const char *title="", 
       const char *xtitle="", const char *ytitle="") : 
  Matrix(nbinsy, Vector(nbinsx)), 
  x_bins(nbinsx), x_min(xmin), x_max(xmax), 
  y_bins(nbinsy), y_min(ymin), y_max(ymax), 
  name(title), 
  x_name(xtitle), y_name(ytitle) 
{}

int H2::fill(double x, double y, double w=1)
{
  int i=x2i(x);
  int j=y2i(y);
  if( i>=0 && i<x_bins && j>=0 && j<y_bins) 
    operator[](j).operator[](i)+=w;
  return y_bins*j+i;
}

double H2::at(double x, double y) const
{
  int i=x2i(x);
  int j=y2i(y);
  if( i>=0 && i<x_bins && j>=0 && j<y_bins) 
    return operator[](j).operator[](i);
  return 0;
}
