/***************************************************************************
    begin                : Wed Apr 10 2002
    copyright            : (C) 2002 by Alexei Soloviev
    email                : solovjev@lynx.jinr.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __DIST_H
#define __DIST_H

#include<vector>
#include<list>
#include<string>


typedef std::vector<double> Vector;
typedef std::vector<Vector> Matrix;

typedef std::list<double> Sample;

class H1 : public Vector
{
  double x_min;
  double x_max;

  std::string name;

  std::string x_name;
  std::string y_name;

 public:
  H1();
  H1(int nbinsx, const double & xmin, const double & xmax, const char *title="",
     const char *xtitle="", const char *ytitle="");
  ~H1(){}

  int fill(double x, double w=1);
  double at(double x) const;

  inline double i2x(int i) const { return x_min+i*xstep(); }
  inline int x2i(double x) const { return int(size()*(x-x_min)/(x_max-x_min)+0.3); }

  inline int xbins() const { return size(); }
  inline double xmin() const { return x_min; }
  inline double xmax() const { return x_max; }
  inline double xstep() const { return (x_max-x_min)/size(); }

  inline const char *title() const { return name.c_str(); }
  inline const char *xtitle() const { return x_name.c_str(); }
  inline const char *ytitle() const { return y_name.c_str(); }

  inline void set_title(const char *t) { name=t; }
  inline void set_xtitle(const char *xt) { x_name=xt; }
  inline void set_ytitle(const char *yt) { y_name=yt; }
};

class H2 : public Matrix
{
  int x_bins; double x_min; double x_max;
  int y_bins; double y_min; double y_max;

  std::string name;

  std::string x_name;
  std::string y_name;

 public:
  H2();
  H2(int nbinsx, const double & xmin, const double & xmax,
     int nbinsy, const double & ymin, const double & ymax,
     const char *title="", 
     const char *xtitle="", const char *ytitle="");
  ~H2(){}

  int fill(double x, double y, double w=1);
  double at(double x, double y) const;

  inline double i2x(int i) const { return x_min+i*xstep(); }
  inline int x2i(double x) const { return int(x_bins*(x-x_min)/(x_max-x_min)+0.3); }

  inline double i2y(int i) const { return y_min+i*ystep(); }
  inline int y2i(double y) const { return int(y_bins*(y-y_min)/(y_max-y_min)+0.3); }

  inline int xbins() const { return x_bins; }
  inline double xmin() const { return x_min; }
  inline double xmax() const { return x_max; }
  inline double xstep() const { return (x_max-x_min)/x_bins; }

  inline int ybins() const { return y_bins; }
  inline double ymin() const { return y_min; }
  inline double ymax() const { return y_max; }
  inline double ystep() const { return (y_max-y_min)/y_bins; }

  inline const char *title() const { return name.c_str(); }
  inline const char *xtitle() const { return x_name.c_str(); }
  inline const char *ytitle() const { return y_name.c_str(); }

  inline void set_title(const char *t) { name=t; }
  inline void set_xtitle(const char *xt) { x_name=xt; }
  inline void set_ytitle(const char *yt) { y_name=yt; }
};

#endif   // __DIST_H
