#ifndef _WAVEDWT_H
#define _WAVEDWT_H

#include "wseries.h"		// Wavelet data classes & methods

#ifdef _USE_ROOT
#include "TObject.h"
#endif

class WaveDWT
#ifdef _USE_ROOT
: public TObject
#endif
{
  public: 

      WSeries *pws;// pointer to WSeries class containing data
        
      int nh;                              // number of filter coefficients
      int MaxLevel;                        // maximal level of decomposition
      int Level;                           // current level of decomposition 
      int IsTree;                          // indicate what kind of decomposition you use: ordinary or binary tree
      virtual void decompose(int,int)=0;   // pure virtual function, should make one step of decomposition
      virtual void reconstruct(int,int)=0; // pure virtual function, should make one step of reconstruction

      WaveDWT(int m=1,int ist=0);                      // Constructor w 
      virtual ~WaveDWT();			       // Destructor

      virtual void t2w(const WSeries&,int=-1);      // forward wavelet transform
      virtual void w2t(WSeries&,int=-1);            // inverse wavelet transform
      virtual void t2w(int=1);                         // forward wavelet transform
      virtual void w2t(int=1);                         // inverse wavelet transform
      void getLayer(WSeries&,int);                  // get layer with given number 
      void putLayer(WSeries&,int);                  // replace layer with given number using data from first parameter
      void getFreqLayer(WSeries&,int);              // the same as getLayer, but get layers in frequency order
      void putFreqLayer(WSeries&,int);              // the same as putLayer, but put layers in frequency order
      virtual int getMaxLevel();                       // get maximal level value
      int getIdA(int,int);                             // get number of array position for given level and layer
      int getIdF(int,int);                             // get frequency order number for given level and layer
      int getIdL(int,int);                             // get number of layer in binary tree for given level and frequency number
      void zeroLayer(int);
      void keepLayer(int);
      void thresholdLayer(int,wavereal);
      void thresholdLayers(wavereal);
      void thresholdLayers(wavereal*);
      wavereal statisticsLayer(int,wavereal&);
      int numberofLayers();

#ifdef _USE_ROOT
  ClassDef(WaveDWT,1)
#endif

};
#endif
