#include "wavefwt.h"
#include <iostream.h>

#ifdef _USE_ROOT
  ClassImp(WaveFWT)
#endif

//--------------------------------------------------------------------
//WaveFWT class is the class implementing Fast Wavelet Transform.

void WaveFWT::decompose(int level,int layer)
{
// Does one step of the decomposition transform.
//

   register int jc=pws->N>>(level+1);
   register int nh2=nh>>1;
   register int jc2=jc<<1;
   register int i,j,j1,l1,i2,i21;
   register int sign;
   
   register wavereal *dataa;
   int ida=getIdA(level,layer);
   dataa=pws->data+ida;
   
   register wavereal *approx=new wavereal[jc];
   register wavereal *detail=new wavereal[jc];
   
   for(i=0;i<jc;i++)
   {
      i2=(i<<1)+nh2;
      i21=(i<<1)-nh2+1;
      approx[i]=0;
      detail[i]=0;
      for(j=0;j<nh;j++)
      {
	 j1=i21+j;
	 l1=i2-j;
	 if(j1<0) {j1+=jc2;}
	 if(j1>=jc2) {j1-=jc2;}
	 if(l1<0) {l1+=jc2;}
	 if(l1>=jc2) {l1-=jc2;}
	 sign=(j&1)==0?-1:1;
	 approx[i]+=h[j]*dataa[j1<<level];
	 detail[i]+=sign*h[j]*dataa[l1<<level];
      }
   }
   
   register int level1=level+1;
   int idaa=getIdA(level1,layer<<1);
   int idad=getIdA(level1,(layer<<1)+1);
   dataa=pws->data+idaa;
   register wavereal *datad=pws->data+idad;
   
   for(i=0;i<jc;i++)
   {
      dataa[i<<level1]=approx[i];
      datad[i<<level1]=detail[i];
   }
   
   delete approx;
   delete detail;
}

void WaveFWT::reconstruct(int level,int layer)
{
// Does one step of the reconstruction transform.

   register int jc=pws->N>>(level+1);
   register int nh2=nh>>1;
   register int jc2=jc<<1;
   register int level1=level+1;
   register int ja=!(nh2&1);
   register int i,j,j1,i2,sign;

   int idaa=getIdA(level1,layer<<1);
   int idad=getIdA(level1,(layer<<1)+1);
   
   register wavereal *dataa=pws->data+idaa;
   register wavereal *datad=pws->data+idad;

   register wavereal *s=new wavereal[jc2];
   
// approximate on this level
   
   for(i=0;i<jc2;i++)
   {
      s[i]=0;
      i2=(i+nh2-1)>>1;
      for(j=0;j<nh2;j++)
      {
	 j1=i2-j;
	 if(j1<0) {j1+=jc;}
	 if(j1>=(jc)) {j1-=jc;}
	 s[i]+=h[(j<<1)+ja]*dataa[j1<<level1];
      }
      ja=!ja;
   }
   
// details on this level
   
   ja=(nh2&1);
   sign=(nh2&1)==0?-1:1;
   
   for(i=0;i<jc2;i++)
   {
      i2=(i-nh2+1)>>1;
      for(j=0;j<nh2;j++)
      {
	 j1=i2+j;
	 if(j1<0) {j1+=jc;}
	 if(j1>=(jc)) {j1-=jc;}
	 s[i]+=sign*h[(j<<1)+ja]*datad[j1<<level1];
      }
      sign=-(sign);
      ja=!ja;
   }
   
   int ida=getIdA(level,layer);
   dataa=pws->data+ida;

   for(i=0;i<jc2;i++)
   {
      dataa[i<<level]=s[i];
   }
   
   delete s;
}

WaveFWT::~WaveFWT()
{
// Destructor.
//
}

WaveFWT::WaveFWT(int ist):WaveDWT(1,ist)
{
// Same as WaveFWT(int ndw), ist indicates the type of transformation: wavelet tree or wavelet binary tree.
//
}

WaveFWT::WaveFWT(int m,int ist):WaveDWT(m,ist)
{
// Two last parameters mean the same as for the  constructor with two parameters.
// First parameter is the number of samples in the wavelet domain series.
//
}























