//________________________________________________________
// WSeries class is the base class for wavelet
// data structures and methods. 

#include "wseries.h"
#include <iostream.h>
#include <fstream.h>

#ifdef _USE_ROOT
  ClassImp(WSeries)
#endif

WSeries::WSeries(){ N = 0; data = NULL;}

WSeries::WSeries(int n)
{ 
  if (n <= 0 ) n = 1;
  data = new wavereal[n];
  N = n;
}


WSeries::~WSeries()
{ 
   delete data; 
}

WSeries& WSeries::operator=(const WSeries &a)
{
  if (this != &a) {
    if (N != a.N) Resize(a.N);
    for (int i=0; i < N; i++) data[i] = a.data[i];
  }
  return *this;
}

WSeries& WSeries::operator=(const WSeries *a)
{ return *this=*a; }


WSeries& WSeries::operator=(const wavereal c)
{
  for (int i=0; i < N; i++) data[i] = c;
  return *this;
}

WSeries& WSeries::operator+=(const WSeries &a)
{
  if (N != a.N) {
   cout <<
   "Error: attempt to use operator += for data with different length.\n";
   cout <<"Data unchanged!\n";
   return *this;
  }
   
  for (int i=0; i < N; i++) data[i] += a.data[i];
  return *this;
}

WSeries& WSeries::operator+=(const WSeries *a)
{ return *this+=*a; }

WSeries& WSeries::operator-=(const WSeries &a)
{
  if (N != a.N) {
    cout <<
    "Error: attempt to use operator -= for data with different length.\n";
    cout <<"Data unchanged!\n";
    return *this;
  }

  for (int i=0; i < N; i++) data[i] -= a.data[i];
  return *this;
}

WSeries& WSeries::operator-=(const WSeries *a) 
{ return *this-=*a; }

WSeries& WSeries::operator+=(const wavereal c)
{
  for (int i=0; i < N; i++) data[i] += c;
  return *this;
}

WSeries& WSeries::operator-=(const wavereal c)
{
  for (int i=0; i < N; i++) data[i] -= c;
  return *this;
}

void WSeries::DumpASCII(const char *fname, int app)
{

  int n=N;
  char *mode = "w";
  if (app == 1) mode = "a";

  FILE *fp;

  if ( (fp = fopen(fname, mode)) == NULL ) {
     cout << "DumpASCII(...) error: cannot open file " << fname <<". \n";
     return;
  };

  for (int i = 0; i < n; i++) fprintf( fp,"%e \n", data[i]);
  fclose(fp); 
}
  
void WSeries::DumpBinary(const char *fname, int app)
{
  int n = N * sizeof(wavereal);
  char *mode = "wb";

  if (app == 1) mode = "ab";

  FILE *fp;

  if ( (fp=fopen(fname, mode)) == NULL ) {
     cout << "DumpBinary(...) error : cannot open file " << fname <<". \n";
     return ;
  }

  fwrite(data, n, 1, fp);
  fclose(fp);
}

void WSeries::DumpShort(const char *fname, int app)
{
  int n = N;
  char *mode = "wb";
  if (app == 1) mode = "ab";

  FILE *fp;
  if ( (fp = fopen(fname, mode)) == NULL ) {
    cout << "DumpShort(...) error : cannot open file " << fname <<". \n";
    return;
  }

  short *dtemp;
  dtemp=new short[n];

  for ( int i=0; i<n; i++ ) dtemp[i]=short(data[i]) ;

  n = n * sizeof(short);

  fwrite(dtemp, n, 1, fp);
  fclose(fp);
  delete dtemp;
}

void WSeries::ReadASCII(const char* fn)
{

  ifstream ifs(fn);
  for(int i=0;i<N;i++) ifs>>data[i];
  ifs.close();
}

void WSeries::ReadBinary(const char *fname)
{

  int n = N * sizeof(wavereal);
  FILE *fp;

  if ( (fp=fopen(fname,"rb")) == NULL ) {
     cout << "ReadBinary(...) error : cannot open file " << fname <<".\n";
     return;
  }

  fread(data, n, 1, fp);
  fclose(fp);
}

void WSeries::ReadShort(const char *fname)
{
  short *dtmp;
  dtmp = new short[N];
  int n = N * sizeof(short);
  FILE *fp;

  if ( (fp=fopen(fname,"rb")) == NULL ) {
     cout << "ReadShort(...) error : cannot open file " << fname <<". \n";
     return;
  }

  cout << "Reading binary record, size="<< n <<"\n";
  fread(dtmp, n, 1, fp);
  for (int i = 0; i < N; i++) data[i] = wavereal(dtmp[i]);
  fclose(fp);
}

void WSeries::Resize(int n)
{
  delete data; 
  data = new wavereal[n];
  N = n;
}

